

class Model:

    class OpenAI:
        GPT_4_0314 = "gpt-4-0314"
        GPT_35_TURBO_0301 = "gpt-3.5-turbo-0301"
        GPT_35_TURBO = "gpt-3.5-turbo"
        GPT_35_TURBO_16K = "gpt-3.5-turbo-16k"
        GPT_4_0125_PREVIEW = "gpt-4-0125-preview"
        GPT_4_TURBO_PREVIEW = "gpt-4-turbo-preview"
        GPT_4_1106_PREVIEW = "gpt-4-1106-preview"
        GPT_4_0613 = "gpt-4-0613"
        GPT_4_VISION_PREVIEW = "gpt-4-vision-preview"
        GPT_4 = "gpt-4"
        GPT_35_TURBO_1106 = "gpt-3.5-turbo-1106"
        GPT_35_TURBO_0613 = "gpt-3.5-turbo-0613"
        GPT_35_TURBO_16K_0613 = "gpt-3.5-turbo-16k-0613"
        GPT_35_TURBO_0125 = "gpt-3.5-turbo-0125"
        GPT_4O = "gpt-4o"
        CHATGPT_4O_LATEST = "chatgpt-4o-latest"
        TEXT_EMBEDDING_ADA_002_V2 = "text-embedding-ada-002-v2"
        TEXT_EMBEDDING_ADA_002 = "text-embedding-ada-002"
        TEXT_EMBEDDING_3_SMALL = "text-embedding-3-small"
        TEXT_EMBEDDING_3_LARGE = "text-embedding-3-large"
        DALL_E_2 = "dall-e-2"
        DALL_E_3 = "dall-e-3"
        GPT_4O_MINI = "gpt-4o-mini"
        GPT_4O_MINI_2024_07_18 = "gpt-4o-mini-2024-07-18"
        GPT_4O_2024_08_06 = "gpt-4o-2024-08-06"
        O1_PREVIEW = "o1-preview"
        O1_MINI = "o1-mini"
        GPT_4O_2024_11_20 = "gpt-4o-2024-11-20"
        O1 = "o1"
        O3_MINI = "o3-mini"

    class Anthropic:
        CLAUDE_3_SONNET_20240229 = "claude-3-sonnet-20240229"
        CLAUDE_3_HAIKU_20240307 = "claude-3-haiku-20240307"
        CLAUDE_3_OPUS_20240229 = "claude-3-opus-20240229"
        CLAUDE_3_5_SONNET_20240620 = "claude-3-5-sonnet-20240620"
        CLAUDE_3_5_SONNET_20241022 = "claude-3-5-sonnet-20241022"
        CLAUDE_3_5_HAIKU_20241022 = "claude-3-5-haiku-20241022"

    class AWSBedrock:
        ANTHROPIC_CLAUDE_3_HAIKU = "anthropic.claude-3-haiku"
        ANTHROPIC_CLAUDE_3_SONNET = "anthropic.claude-3-sonnet"
        ANTHROPIC_CLAUDE_3_OPUS = "anthropic.claude-3-opus"
        ANTHROPIC_CLAUDE_3_5_SONNET = "anthropic.claude-3.5-sonnet"
        ANTHROPIC_CLAUDE_3_5_SONNET_V2 = "anthropic.claude-3.5-sonnet-v2"
        META_LLAMA3_8B = "meta.llama3-8b"
        META_LLAMA3_70B = "meta.llama3-70b"
        AMAZON_TITAN_LITE_V1 = "amazon.titan-lite-v1"
        AMAZON_TITAN_EXPRESS_V1 = "amazon.titan-express-v1"
        AMAZON_TITAN_EMBED_V2 = "amazon.titan-embed-v2"
        COHERE_COMMAND = "cohere.command"
        COHERE_EMBED_ENGLISH = "cohere.embed-english"
        COHERE_EMBED_MULTILINGUAL = "cohere.embed-multilingual"
        META_LLAMA3_1_70B = "meta.llama3-1-70b"
        META_LLAMA3_1_405B = "meta.llama3-1-405b"
        META_LLAMA3_2_1B = "meta.llama3-2-1b"
        META_LLAMA3_2_3B = "meta.llama3-2-3b"
        META_LLAMA3_2_11B = "meta.llama3-2-11b"
        META_LLAMA3_2_90B = "meta.llama3-2-90b"
        ANTHROPIC_CLAUDE_3_5_HAIKU = "anthropic.claude-3.5-haiku"
        AMAZON_NOVA_MICRO_V1_0 = "amazon.nova-micro-v1:0"
        AMAZON_NOVA_LITE_V1_0 = "amazon.nova-lite-v1:0"
        AMAZON_NOVA_PRO_V1_0 = "amazon.nova-pro-v1:0"
        COHERE_RERANK_V3_5 = "cohere.rerank-v3.5"
        AMAZON_RERANK_V1 = "amazon.rerank-v1"
        AMAZON_TITAN_EMBED_IMAGE_V1 = "amazon.titan-embed-image-v1"

    class AzureOpenAI:
        GPT_35_TURBO_16K = "gpt-3.5-turbo-16k"
        GPT_4 = "gpt-4"
        GPT_4O_MINI = "gpt-4o-mini"
        GPT_4O = "gpt-4o"
        TEXT_EMBEDDING_ADA_002 = "text-embedding-ada-002"
        TEXT_EMBEDDING_3_SMALL = "text-embedding-3-small"
        DEEPSEEK_R1 = "deepseek-r1"

    class Cerebras:
        LLAMA3_1_8B = "llama3.1-8b"
        LLAMA3_1_70B = "llama3.1-70b"
        LLAMA3_3_70B = "llama-3.3-70b"

    class CohereModels:
        COMMAND_R = "command-r"
        COMMAND_R_08_2024 = "command-r-08-2024"
        COMMAND_R_PLUS = "command-r-plus"
        COMMAND_R_PLUS_08_2024 = "command-r-plus-08-2024"
        COMMAND_R7B_12_2024 = "command-r7b-12-2024"
        EMBED_ENGLISH_V3_0 = "embed-english-v3.0"
        EMBED_MULTILINGUAL_V3_0 = "embed-multilingual-v3.0"
        EMBED_ENGLISH_LIGHT_V3_0 = "embed-english-light-v3.0"
        EMBED_MULTILINGUAL_LIGHT_V3_0 = "embed-multilingual-light-v3.0"
        RERANK_V3_5 = "rerank-v3.5"

    class Gemini:
        GEMINI_1_5_FLASH_LATEST = "gemini-1.5-flash-latest"
        GEMINI_1_5_FLASH_EXP_0827 = "gemini-1.5-flash-exp-0827"
        GEMINI_1_5_FLASH_8B_EXP_0827 = "gemini-1.5-flash-8b-exp-0827"
        GEMINI_1_5_PRO_LATEST = "gemini-1.5-pro-latest"
        GEMINI_1_5_PRO_EXP_0801 = "gemini-1.5-pro-exp-0801"
        GEMINI_1_5_PRO_EXP_0827 = "gemini-1.5-pro-exp-0827"
        GEMINI_EXP_1114 = "gemini-exp-1114"
        GEMINI_EXP_1121 = "gemini-exp-1121"
        GEMINI_EXP_1206 = "gemini-exp-1206"
        GEMINI_2_0_FLASH_EXP = "gemini-2.0-flash-exp"
        GEMINI_2_0_FLASH_THINKING_EXP_1219 = "gemini-2.0-flash-thinking-exp-1219"
        GEMINI_2_0_FLASH_THINKING_EXP_01_21 = "gemini-2.0-flash-thinking-exp-01-21"
        GEMINI_2_0_FLASH_LITE_PREVIEW = "gemini-2.0-flash-lite-preview"
        GEMINI_2_0_PRO_EXP = "gemini-2.0-pro-exp"

    class VertexAI:
        GEMINI_1_5_FLASH_PREVIEW_0514 = "gemini-1.5-flash-preview-0514"
        GEMINI_1_5_PRO_PREVIEW_0409 = "gemini-1.5-pro-preview-0409"
        GEMINI_1_5_PRO_PREVIEW_0514 = "gemini-1.5-pro-preview-0514"
        GEMINI_1_0_PRO = "gemini-1.0-pro"
        GEMINI_PRO = "gemini-pro"
        CLAUDE_3_SONNET_20240229 = "claude-3-sonnet-20240229"
        CLAUDE_3_HAIKU_20240307 = "claude-3-haiku-20240307"
        CLAUDE_3_OPUS_20240229 = "claude-3-opus-20240229"
        CLAUDE_3_5_SONNET_20240620 = "claude-3-5-sonnet-20240620"
        GEMINI_1_5_FLASH = "gemini-1.5-flash"
        GEMINI_1_5_PRO = "gemini-1.5-pro"
        GEMINI_1_5_PRO_LATEST = "gemini-1.5-pro-latest"
        GEMINI_1_5_PRO_EXP_0801 = "gemini-1.5-pro-exp-0801"
        TEXT_EMBEDDING_004 = "text-embedding-004"
        TEXT_EMBEDDING_GECKO = "textembedding-gecko"
        TEXT_EMBEDDING_GECKO_MULTILINGUAL = "textembedding-gecko-multilingual"
        META_LLAMA_3_2_90B_VISION_INSTRUCT_MAAS = "meta.llama-3.2-90b-vision-instruct-maas"
        CLAUDE_3_5_SONNET_V2_20241022 = "claude-3-5-sonnet-v2-20241022"
        GEMINI_1_5_PRO_002 = "gemini-1.5-pro-002"
        GEMINI_1_5_PRO_002_LB = "gemini-1.5-pro-002-lb"
        GEMINI_1_5_FLASH_002 = "gemini-1.5-flash-002"
        GEMINI_1_5_FLASH_002_LB = "gemini-1.5-flash-002-lb"
        CLAUDE_3_5_HAIKU_20241022 = "claude-3-5-haiku-20241022"
        META_LLAMA_3_1_405B_INSTRUCT_MAAS = "meta.llama-3.1-405b-instruct-maas"
        META_LLAMA_3_1_70B_INSTRUCT_MAAS = "meta.llama-3.1-70b-instruct-maas"
        META_LLAMA_3_1_8B_INSTRUCT_MAAS = "meta.llama-3.1-8b-instruct-maas"
        GEMINI_2_0_FLASH_EXP = "gemini-2.0-flash-exp"
        CODESTRAL_2501 = "codestral-2501"
        GEMINI_2_0_FLASH_THINKING_EXP_01_21 = "gemini-2.0-flash-thinking-exp-01-21"
        GEMINI_2_0_FLASH_001 = "gemini-2.0-flash-001"

    class Groq:
        LLAMA_3_1_70B_VERSATILE = "llama-3.1-70b-versatile"
        LLAMA_3_1_405B_REASONING = "llama-3.1-405b-reasoning"
        LLAMA_3_2_1B_PREVIEW = "llama-3.2-1b-preview"
        LLAMA_3_2_3B_PREVIEW = "llama-3.2-3b-preview"
        LLAMA_3_2_11B_VISION_PREVIEW = "llama-3.2-11b-vision-preview"
        LLAMA_3_2_90B_VISION_PREVIEW = "llama-3.2-90b-vision-preview"
        LLAMA_3_1_8B_INSTANT = "llama-3.1-8b-instant"
        MIXTRAL_8X7B_32768 = "mixtral-8x7b-32768"
        LLAMA_3_3_70B_VERSATILE = "llama-3.3-70b-versatile"
        DEEPSEEK_R1_DISTILL_LLAMA_70B = "deepseek-r1-distill-llama-70b"

    class Nvidia:
        NEMOTRON_MINI_4B_INSTRUCT = "nvidia.nemotron-mini-4b-instruct"
        META_LLAMA_3_1_8B_INSTRUCT = "meta.llama-3.1-8b-instruct"
        META_LLAMA_3_1_70B_INSTRUCT = "meta.llama-3.1-70b-instruct"
        META_LLAMA_3_1_405B_INSTRUCT = "meta.llama-3.1-405b-instruct"
        META_LLAMA_3_2_1B_INSTRUCT = "meta.llama-3.2-1b-instruct"
        META_LLAMA_3_2_3B_INSTRUCT = "meta.llama-3.2-3b-instruct"
        BAAI_BGE_M3 = "baai.bge-m3"
        NV_EMBED_V1 = "nv-embed-v1"
        NVCLIP = "nvclip"
        EMBED_QA_4 = "embed-qa-4"
        NV_EMBEDQA_E5_V5 = "nv-embedqa-e5-v5"
        SNOWFLAKE_ARCTIC_EMBED_L = "snowflake.arctic-embed-l"
        NV_EMBEDQA_MISTRAL_7B_V2 = "nv-embedqa-mistral-7b-v2"
        LLAMA_3_1_NEMOTRON_70B_INSTRUCT = "llama-3.1-nemotron-70b-instruct"
        DEEPSEEK_AI_DEEPSEEK_R1 = "deepseek-ai-deepseek-r1"

    class SambaNova:
        QWEN_2_5_72B_INSTRUCT = "Qwen2.5-72B-Instruct"
        QWEN_2_5_CODER_32B_INSTRUCT = "Qwen2.5-Coder-32B-Instruct"
        META_LLAMA_3_1_8B_INSTRUCT = "Meta-Llama-3.1-8B-Instruct"
        META_LLAMA_3_1_70B_INSTRUCT = "Meta-Llama-3.1-70B-Instruct"
        META_LLAMA_3_1_405B_INSTRUCT = "Meta-Llama-3.1-405B-Instruct"
        META_LLAMA_3_2_1B_INSTRUCT = "Meta-Llama-3.2-1B-Instruct"
        META_LLAMA_3_2_3B_INSTRUCT = "Meta-Llama-3.2-3B-Instruct"
        LLAMA_3_2_11B_VISION_INSTRUCT = "Llama-3.2-11B-Vision-Instruct"
        LLAMA_3_2_90B_VISION_INSTRUCT = "Llama-3.2-90B-Vision-Instruct"
        META_LLAMA_3_3_70B_INSTRUCT = "Meta-Llama-3.3-70B-Instruct"



import json
from pygeai.cli.commands import Command, Option, ArgumentsEnum
from pygeai.cli.commands.builders import build_help_text
from pygeai.cli.commands.common import get_boolean_value
from pygeai.cli.texts.help import OMNI_HELP_TEXT
from pygeai.core.common.exceptions import MissingRequirementException, WrongArgumentError
from pygeai.core.omni.clients import OmniParserClient
from pygeai.core.utils.console import Console


def show_help():
    """
    Displays help text in stdout
    """
    help_text = build_help_text(omni_commands, OMNI_HELP_TEXT)
    Console.write_stdout(help_text)


def parse_file(option_list: list):
    start_page = None
    end_page = None
    hi_res_mode = None
    dialogue = None
    cache = None

    for option_flag, option_arg in option_list:
        if option_flag.name == "start_page":
            try:
                start_page = int(option_arg)
            except (ValueError, TypeError):
                raise WrongArgumentError("start_page must be an integer")
        elif option_flag.name == "end_page":
            try:
                end_page = int(option_arg)
            except (ValueError, TypeError):
                raise WrongArgumentError("end_page must be an integer")
        elif option_flag.name == "hi_res_mode":
            hi_res_mode = get_boolean_value(option_arg)
        elif option_flag.name == "dialogue":
            dialogue = get_boolean_value(option_arg)
        elif option_flag.name == "cache":
            cache = get_boolean_value(option_arg)

    opts = {opt.name: arg for opt, arg in option_list}
    file_path = opts.get('input')
    structure = opts.get('structure')
    output_format = opts.get('output_format')
    password = opts.get('password')

    if not file_path:
        raise MissingRequirementException("Cannot process file without specifying --input parameter")

    client = OmniParserClient()
    result = client.process_file(
        file_path=file_path,
        start_page=start_page,
        end_page=end_page,
        hi_res_mode=hi_res_mode,
        structure=structure,
        dialogue=dialogue,
        output_format=output_format,
        password=password,
        cache=cache if cache is not None else False
    )

    output = {
        "status": result.get("status"),
        "request_id": result.get("requestId"),
        "parts_count": len(result.get("parts", [])),
        "usage": result.get("usage"),
        "parts": []
    }

    for part in result.get("parts", []):
        part_info = {
            "type": part.get("type"),
            "text_length": len(part.get("text", ""))
        }
        if part.get("page") is not None:
            part_info["page"] = part.get("page")
        if part.get("midTime") is not None:
            part_info["mid_time"] = part.get("midTime")

        part_info["text_preview"] = part.get("text", "")[:200] + ("..." if len(part.get("text", "")) > 200 else "")
        output["parts"].append(part_info)

    Console.write_stdout(json.dumps(output, indent=2))


parse_file_options = [
    Option(
        "input",
        ["--input", "-i"],
        "File path to process (required)",
        True
    ),
    Option(
        "start_page",
        ["--start-page", "--sp"],
        "integer: Start page number for processing (1-based, optional)",
        True
    ),
    Option(
        "end_page",
        ["--end-page", "--ep"],
        "integer: End page number for processing (inclusive, use 0 for until end, optional)",
        True
    ),
    Option(
        "hi_res_mode",
        ["--hi-res", "--hr"],
        "Enable high-resolution OCR/vision mode for complex scans (1/0, optional)",
        True
    ),
    Option(
        "structure",
        ["--structure", "--st"],
        "string: Structure format like 'table' for spreadsheet/CSV sources (optional)",
        True
    ),
    Option(
        "dialogue",
        ["--dialogue", "--dl"],
        "For audio/video: enable dialogue transcription mode (1/0, optional)",
        True
    ),
    Option(
        "output_format",
        ["--output-format", "--of"],
        "string: Output format - plainText (default), markdown, or html (optional)",
        True
    ),
    Option(
        "password",
        ["--password", "--pwd"],
        "string: Password for password-protected files (optional)",
        True
    ),
    Option(
        "cache",
        ["--cache"],
        "Enable X-Saia-Cache-Enabled to cache processing results. 1 to enable, 0 to disable (default: 0)",
        True
    ),
]


omni_commands = [
    Command(
        "help",
        ["help", "h"],
        "Display help text",
        show_help,
        ArgumentsEnum.NOT_AVAILABLE,
        [],
        []
    ),
    Command(
        "parse",
        ["parse", "p"],
        "Process file with omni-parser",
        parse_file,
        ArgumentsEnum.REQUIRED,
        [],
        parse_file_options
    ),
]



from unittest import TestCase
import tempfile
import os
from pygeai.lab.spec.loader import JSONLoader
from pygeai.core.common.exceptions import InvalidPathException, InvalidJSONException


class TestJSONLoader(TestCase):
    """
    python -m unittest pygeai.tests.lab.spec.test_loader.TestJSONLoader
    """

    def setUp(self):
        # Create temporary directory for test files
        self.temp_dir = tempfile.TemporaryDirectory()
        self.valid_file_path = os.path.join(self.temp_dir.name, 'valid_file.json')
        self.invalid_file_path = os.path.join(self.temp_dir.name, 'invalid_file.json')

        # Create valid JSON file
        with open(self.valid_file_path, 'w') as f:
            f.write('{"key": "value"}')

        # Create invalid JSON file
        with open(self.invalid_file_path, 'w') as f:
            f.write('{key: value}')  # Invalid JSON: missing quotes around key

    def tearDown(self):
        # Clean up temporary directory and files
        self.temp_dir.cleanup()

    def test_load_data_valid_json(self):
        """Test loading a valid JSON file."""
        data = JSONLoader.load_data(self.valid_file_path)
        self.assertIsInstance(data, dict)
        self.assertEqual(data, {"key": "value"})

    def test_load_data_invalid_json(self):
        """Test loading an invalid JSON file."""
        with self.assertRaises(InvalidJSONException) as cm:
            JSONLoader.load_data(self.invalid_file_path)
        self.assertIn(f"File {self.invalid_file_path} doesn't contain valid JSON.", str(cm.exception))

    def test_load_data_non_existent_file(self):
        """Test loading a non-existent file."""
        non_existent_path = os.path.join(self.temp_dir.name, 'non_existent.json')
        with self.assertRaises(InvalidPathException) as cm:
            JSONLoader.load_data(non_existent_path)
        self.assertIn(f"File {non_existent_path} doesn't exist", str(cm.exception))

    def test_load_data_empty_file(self):
        """Test loading an empty JSON file."""
        empty_file_path = os.path.join(self.temp_dir.name, 'empty_file.json')
        with open(empty_file_path, 'w') as f:
            f.write('')
        with self.assertRaises(InvalidJSONException) as cm:
            JSONLoader.load_data(empty_file_path)
        self.assertIn(f"File {empty_file_path} doesn't contain valid JSON.", str(cm.exception))


import json
from pathlib import Path

from pygeai import logger
from pygeai.core.common.exceptions import InvalidPathException, InvalidJSONException


class JSONLoader:

    @classmethod
    def load_data(cls, file_path: str):
        data = {}
        file = Path(file_path)
        if not file.exists():
            raise InvalidPathException(f"File {file_path} doesn't exist")

        with open(file_path, 'r') as f:
            try:
                data = json.load(f)
            except json.JSONDecodeError as e:
                logger.error(f"The file doesn't contain a valid JSON: {e}")
                raise InvalidJSONException(f"File {file_path} doesn't contain valid JSON.")

        return data



from unittest import TestCase
import uuid
from pygeai.lab.managers import AILabManager
from pygeai.lab.models import AgenticProcess, KnowledgeBase, AgenticActivity, ArtifactSignal, UserSignal, Event, SequenceFlow
from pygeai.core.common.exceptions import InvalidAPIResponseException, MissingRequirementException

ai_lab_manager: AILabManager

class TestAILabDeleteProcessIntegration(TestCase):  

    def setUp(self):
        self.ai_lab_manager = AILabManager()
        

    def __create_process(self):
        """
        Helper to create a process with the current ai_lab_manager.
        """
        unique_key = str(uuid.uuid4())
        process = AgenticProcess(
            key="product_def",
            name=f"Test Process {unique_key[:8]}",
            description="This is a sample process",
            kb=KnowledgeBase(name="basic-sample", artifact_type_name=["sample-artifact"]),
            agentic_activities=[
                AgenticActivity(key="activityOne", name="First Step", task_name="basic-task", agent_name="GoogleSummarizer2", agent_revision_id=0)
            ],
            artifact_signals=[
                ArtifactSignal(key="artifact.upload.1", name="artifact.upload", handling_type="C", artifact_type_name=["sample-artifact"])
            ],
            user_signals=[
                UserSignal(key="signal_done", name="process-completed")
            ],
            start_event=Event(key="artifact.upload.1", name="artifact.upload"),
            end_event=Event(key="end", name="Done"),
            sequence_flows=[
                SequenceFlow(key="step1", source_key="artifact.upload.1", target_key="activityOne"),
                SequenceFlow(key="step2", source_key="activityOne", target_key="signal_done"),
                SequenceFlow(key="stepEnd", source_key="signal_done", target_key="end")
            ]
        )

        return self.ai_lab_manager.create_process(
            process=process,
            automatic_publish=False
        )
    

    def __delete_process(self, process_id: str = None, process_name: str = None):
        return self.ai_lab_manager.delete_process(
            process_id=process_id,
            process_name=process_name
        )
    

    def test_delete_process_by_id(self):         
        created_process = self.__create_process()     
        deleted_process = self.__delete_process(process_id=created_process.id)
        print(deleted_process)

        self.assertEqual(
            deleted_process.content,
            "Process deleted successfully",
            "Expected confirmation message after deletion"            
        )


    def test_delete_process_by_name(self):         
        created_process = self.__create_process()     
        deleted_process = self.__delete_process(process_id="",process_name=created_process.name)

        self.assertEqual(
            deleted_process.content,
            "Process deleted successfully",
            "Expected confirmation message after deletion"            
        )


    def test_delete_process_no_parameters(self):
        with self.assertRaises(MissingRequirementException) as exception:
            self.__delete_process()
        self.assertIn(
            "Either process_id or process_name must be provided",
            str(exception.exception),
            "Expected error message for missing parameters"
        )


    def test_delete_process_invalid_process_id(self):
        invalid_id = "0026e53d-ea78-4cac-af9f-12650invalid"
        with self.assertRaises(InvalidAPIResponseException) as exception:
            self.__delete_process(process_id=invalid_id)

        self.assertIn(
            f"Unable to delete process {invalid_id}",
            str(exception.exception),
            "Expected error message for invalid process id"
        )


    def test_delete_process_invalid_process_name(self):
        invalid_name = "NonExistentProcessName123"
        with self.assertRaises(InvalidAPIResponseException) as exception:
            self.__delete_process(process_name=invalid_name)

        self.assertIn(
            f"Unable to delete process {invalid_name}",
            str(exception.exception),
            "Expected error message for invalid process name"
        )


from pygeai import logger
from pygeai.admin.clients import AdminClient
from pygeai.analytics.endpoints import (
    GET_AGENTS_CREATED_AND_MODIFIED_V1, GET_AGENTS_CREATED_AND_MODIFIED_PER_DAY_V1,
    GET_FLOWS_CREATED_AND_MODIFIED_V1, GET_FLOWS_CREATED_AND_MODIFIED_PER_DAY_V1,
    GET_PROCESSES_CREATED_AND_MODIFIED_V1, GET_AGENT_USAGE_PER_USER_V1,
    GET_AVERAGE_COST_PER_REQUEST_V1, GET_AVERAGE_COST_PER_USER_V1,
    GET_AVERAGE_COST_PER_USER_PER_DATE_V1, GET_AVERAGE_REQUEST_TIME_V1,
    GET_AVERAGE_REQUESTS_PER_DAY_V1, GET_AVERAGE_REQUESTS_PER_USER_V1,
    GET_AVERAGE_REQUESTS_PER_USER_PER_DATE_V1, GET_AVERAGE_TOKENS_PER_REQUEST_V1,
    GET_AVERAGE_USERS_PER_AGENT_V1, GET_AVERAGE_USERS_PER_PROJECT_V1,
    GET_NUMBER_OF_TOKENS_V1, GET_NUMBER_OF_TOKENS_PER_AGENT_V1,
    GET_NUMBER_OF_TOKENS_PER_DAY_V1, GET_OVERALL_ERROR_RATE_V1,
    GET_TOP_10_AGENTS_BY_REQUESTS_V1, GET_TOP_10_AGENTS_BY_TOKENS_V1,
    GET_TOP_10_USERS_BY_COST_V1, GET_TOP_10_USERS_BY_REQUESTS_V1,
    GET_TOTAL_ACTIVE_AGENTS_V1, GET_TOTAL_ACTIVE_PROJECTS_V1,
    GET_TOTAL_ACTIVE_USERS_V1, GET_TOTAL_COST_V1, GET_TOTAL_COST_PER_DAY_V1,
    GET_TOTAL_REQUEST_TIME_V1, GET_TOTAL_REQUESTS_V1, GET_TOTAL_REQUESTS_PER_DAY_V1,
    GET_TOTAL_REQUESTS_WITH_ERROR_V1, GET_TOTAL_TOKENS_V1
)
from pygeai.core.base.clients import BaseClient
from pygeai.core.common.exceptions import APIError
from pygeai.core.utils.validators import validate_status_code
from pygeai.core.utils.parsers import parse_json_response


class AnalyticsClient(BaseClient):

    def __init__(self, api_key: str = None, base_url: str = None, alias: str = None, *,
                 access_token: str = None, project_id: str = None, organization_id: str = None):
        super().__init__(api_key, base_url, alias, access_token=access_token, project_id=project_id)
        ids = self.__get_project_and_organization_ids(api_key, base_url, alias, project_id, organization_id)
        self.project_id = ids.get("projectId")
        self.organization_id = ids.get("organizationId")
        if self.project_id and not self.api_service.project_id:
            self.api_service.project_id = self.project_id

    def __get_project_and_organization_ids(self, api_key: str = None, base_url: str = None, alias: str = None,
                                           project_id: str = None, organization_id: str = None):
        if project_id and organization_id:
            return {"projectId": project_id, "organizationId": organization_id}
        
        response = None
        try:
            response = AdminClient(api_key=api_key, base_url=base_url, alias=alias).validate_api_token()
            return {
                "projectId": project_id if project_id else response.get("projectId"),
                "organizationId": organization_id if organization_id else response.get("organizationId")
            }
        except Exception as e:
            logger.error(f"Error retrieving project_id and organization_id from GEAI. Response: {response}: {e}")
            raise APIError(f"Error retrieving project_id and organization_id from GEAI: {e}")

    def __build_headers(self):
        return {
            "Authorization": f"Bearer {self.api_service.token}",
            "OrganizationId": self.organization_id,
            "ProjectId": self.project_id
        }

    def get_agents_created_and_modified(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AGENTS_CREATED_AND_MODIFIED_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get agents created and modified")

    def get_agents_created_and_modified_per_day(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AGENTS_CREATED_AND_MODIFIED_PER_DAY_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get agents created and modified per day")

    def get_flows_created_and_modified(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_FLOWS_CREATED_AND_MODIFIED_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get flows created and modified")

    def get_flows_created_and_modified_per_day(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_FLOWS_CREATED_AND_MODIFIED_PER_DAY_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get flows created and modified per day")

    def get_processes_created_and_modified(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_PROCESSES_CREATED_AND_MODIFIED_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get processes created and modified")

    def get_agent_usage_per_user(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AGENT_USAGE_PER_USER_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get agent usage per user")

    def get_average_cost_per_request(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AVERAGE_COST_PER_REQUEST_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get average cost per request")

    def get_average_cost_per_user(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AVERAGE_COST_PER_USER_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get average cost per user")

    def get_average_cost_per_user_per_date(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AVERAGE_COST_PER_USER_PER_DATE_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get average cost per user per date")

    def get_average_request_time(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AVERAGE_REQUEST_TIME_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get average request time")

    def get_average_requests_per_day(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AVERAGE_REQUESTS_PER_DAY_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get average requests per day")

    def get_average_requests_per_user(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AVERAGE_REQUESTS_PER_USER_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get average requests per user")

    def get_average_requests_per_user_per_date(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AVERAGE_REQUESTS_PER_USER_PER_DATE_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get average requests per user per date")

    def get_average_tokens_per_request(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AVERAGE_TOKENS_PER_REQUEST_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get average tokens per request")

    def get_average_users_per_agent(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AVERAGE_USERS_PER_AGENT_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get average users per agent")

    def get_average_users_per_project(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_AVERAGE_USERS_PER_PROJECT_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get average users per project")

    def get_number_of_tokens(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_NUMBER_OF_TOKENS_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get number of tokens")

    def get_number_of_tokens_per_agent(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_NUMBER_OF_TOKENS_PER_AGENT_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get number of tokens per agent")

    def get_number_of_tokens_per_day(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_NUMBER_OF_TOKENS_PER_DAY_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get number of tokens per day")

    def get_overall_error_rate(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_OVERALL_ERROR_RATE_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get overall error rate")

    def get_top_10_agents_by_requests(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOP_10_AGENTS_BY_REQUESTS_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get top 10 agents by requests")

    def get_top_10_agents_by_tokens(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOP_10_AGENTS_BY_TOKENS_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get top 10 agents by tokens")

    def get_top_10_users_by_cost(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOP_10_USERS_BY_COST_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get top 10 users by cost")

    def get_top_10_users_by_requests(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOP_10_USERS_BY_REQUESTS_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get top 10 users by requests")

    def get_total_active_agents(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOTAL_ACTIVE_AGENTS_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get total active agents")

    def get_total_active_projects(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOTAL_ACTIVE_PROJECTS_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get total active projects")

    def get_total_active_users(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOTAL_ACTIVE_USERS_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get total active users")

    def get_total_cost(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOTAL_COST_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get total cost")

    def get_total_cost_per_day(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOTAL_COST_PER_DAY_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get total cost per day")

    def get_total_request_time(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOTAL_REQUEST_TIME_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get total request time")

    def get_total_requests(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOTAL_REQUESTS_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get total requests")

    def get_total_requests_per_day(
            self,
            start_date: str,
            end_date: str,
            agent_name: str = None
    ) -> dict:
        params = {"startDate": start_date, "endDate": end_date}
        if agent_name:
            params["agentName"] = agent_name
        response = self.api_service.get(
            endpoint=GET_TOTAL_REQUESTS_PER_DAY_V1,
            headers=self.__build_headers(),
            params=params
        )
        validate_status_code(response)
        return parse_json_response(response, "get total requests per day")

    def get_total_requests_with_error(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOTAL_REQUESTS_WITH_ERROR_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get total requests with error")

    def get_total_tokens(
            self,
            start_date: str,
            end_date: str
    ) -> dict:
        response = self.api_service.get(
            endpoint=GET_TOTAL_TOKENS_V1,
            headers=self.__build_headers(),
            params={"startDate": start_date, "endDate": end_date}
        )
        validate_status_code(response)
        return parse_json_response(response, "get total tokens")



"""Backwards compatibility layer for legacy A2A clients."""

import warnings

from collections.abc import AsyncGenerator
from typing import Any

import httpx

from a2a.client.errors import A2AClientJSONRPCError
from a2a.client.middleware import ClientCallContext, ClientCallInterceptor
from a2a.client.transports.jsonrpc import JsonRpcTransport
from a2a.types import (
    AgentCard,
    CancelTaskRequest,
    CancelTaskResponse,
    CancelTaskSuccessResponse,
    GetTaskPushNotificationConfigParams,
    GetTaskPushNotificationConfigRequest,
    GetTaskPushNotificationConfigResponse,
    GetTaskPushNotificationConfigSuccessResponse,
    GetTaskRequest,
    GetTaskResponse,
    GetTaskSuccessResponse,
    JSONRPCErrorResponse,
    SendMessageRequest,
    SendMessageResponse,
    SendMessageSuccessResponse,
    SendStreamingMessageRequest,
    SendStreamingMessageResponse,
    SendStreamingMessageSuccessResponse,
    SetTaskPushNotificationConfigRequest,
    SetTaskPushNotificationConfigResponse,
    SetTaskPushNotificationConfigSuccessResponse,
    TaskIdParams,
    TaskResubscriptionRequest,
)


class A2AClient:
    """[DEPRECATED] Backwards compatibility wrapper for the JSON-RPC client."""

    def __init__(
        self,
        httpx_client: httpx.AsyncClient,
        agent_card: AgentCard | None = None,
        url: str | None = None,
        interceptors: list[ClientCallInterceptor] | None = None,
    ):
        warnings.warn(
            'A2AClient is deprecated and will be removed in a future version. '
            'Use ClientFactory to create a client with a JSON-RPC transport.',
            DeprecationWarning,
            stacklevel=2,
        )
        self._transport = JsonRpcTransport(
            httpx_client, agent_card, url, interceptors
        )

    async def send_message(
        self,
        request: SendMessageRequest,
        *,
        http_kwargs: dict[str, Any] | None = None,
        context: ClientCallContext | None = None,
    ) -> SendMessageResponse:
        """Sends a non-streaming message request to the agent.

        Args:
            request: The `SendMessageRequest` object containing the message and configuration.
            http_kwargs: Optional dictionary of keyword arguments to pass to the
                underlying httpx.post request.
            context: The client call context.

        Returns:
            A `SendMessageResponse` object containing the agent's response (Task or Message) or an error.

        Raises:
            A2AClientHTTPError: If an HTTP error occurs during the request.
            A2AClientJSONError: If the response body cannot be decoded as JSON or validated.
        """
        if not context and http_kwargs:
            context = ClientCallContext(state={'http_kwargs': http_kwargs})

        try:
            result = await self._transport.send_message(
                request.params, context=context
            )
            return SendMessageResponse(
                root=SendMessageSuccessResponse(
                    id=request.id, jsonrpc='2.0', result=result
                )
            )
        except A2AClientJSONRPCError as e:
            return SendMessageResponse(JSONRPCErrorResponse(error=e.error))

    async def send_message_streaming(
        self,
        request: SendStreamingMessageRequest,
        *,
        http_kwargs: dict[str, Any] | None = None,
        context: ClientCallContext | None = None,
    ) -> AsyncGenerator[SendStreamingMessageResponse, None]:
        """Sends a streaming message request to the agent and yields responses as they arrive.

        This method uses Server-Sent Events (SSE) to receive a stream of updates from the agent.

        Args:
            request: The `SendStreamingMessageRequest` object containing the message and configuration.
            http_kwargs: Optional dictionary of keyword arguments to pass to the
                underlying httpx.post request. A default `timeout=None` is set but can be overridden.
            context: The client call context.

        Yields:
            `SendStreamingMessageResponse` objects as they are received in the SSE stream.
            These can be Task, Message, TaskStatusUpdateEvent, or TaskArtifactUpdateEvent.

        Raises:
            A2AClientHTTPError: If an HTTP or SSE protocol error occurs during the request.
            A2AClientJSONError: If an SSE event data cannot be decoded as JSON or validated.
        """
        if not context and http_kwargs:
            context = ClientCallContext(state={'http_kwargs': http_kwargs})

        async for result in self._transport.send_message_streaming(
            request.params, context=context
        ):
            yield SendStreamingMessageResponse(
                root=SendStreamingMessageSuccessResponse(
                    id=request.id, jsonrpc='2.0', result=result
                )
            )

    async def get_task(
        self,
        request: GetTaskRequest,
        *,
        http_kwargs: dict[str, Any] | None = None,
        context: ClientCallContext | None = None,
    ) -> GetTaskResponse:
        """Retrieves the current state and history of a specific task.

        Args:
            request: The `GetTaskRequest` object specifying the task ID and history length.
            http_kwargs: Optional dictionary of keyword arguments to pass to the
                underlying httpx.post request.
            context: The client call context.

        Returns:
            A `GetTaskResponse` object containing the Task or an error.

        Raises:
            A2AClientHTTPError: If an HTTP error occurs during the request.
            A2AClientJSONError: If the response body cannot be decoded as JSON or validated.
        """
        if not context and http_kwargs:
            context = ClientCallContext(state={'http_kwargs': http_kwargs})
        try:
            result = await self._transport.get_task(
                request.params, context=context
            )
            return GetTaskResponse(
                root=GetTaskSuccessResponse(
                    id=request.id, jsonrpc='2.0', result=result
                )
            )
        except A2AClientJSONRPCError as e:
            return GetTaskResponse(root=JSONRPCErrorResponse(error=e.error))

    async def cancel_task(
        self,
        request: CancelTaskRequest,
        *,
        http_kwargs: dict[str, Any] | None = None,
        context: ClientCallContext | None = None,
    ) -> CancelTaskResponse:
        """Requests the agent to cancel a specific task.

        Args:
            request: The `CancelTaskRequest` object specifying the task ID.
            http_kwargs: Optional dictionary of keyword arguments to pass to the
                underlying httpx.post request.
            context: The client call context.

        Returns:
            A `CancelTaskResponse` object containing the updated Task with canceled status or an error.

        Raises:
            A2AClientHTTPError: If an HTTP error occurs during the request.
            A2AClientJSONError: If the response body cannot be decoded as JSON or validated.
        """
        if not context and http_kwargs:
            context = ClientCallContext(state={'http_kwargs': http_kwargs})
        try:
            result = await self._transport.cancel_task(
                request.params, context=context
            )
            return CancelTaskResponse(
                root=CancelTaskSuccessResponse(
                    id=request.id, jsonrpc='2.0', result=result
                )
            )
        except A2AClientJSONRPCError as e:
            return CancelTaskResponse(JSONRPCErrorResponse(error=e.error))

    async def set_task_callback(
        self,
        request: SetTaskPushNotificationConfigRequest,
        *,
        http_kwargs: dict[str, Any] | None = None,
        context: ClientCallContext | None = None,
    ) -> SetTaskPushNotificationConfigResponse:
        """Sets or updates the push notification configuration for a specific task.

        Args:
            request: The `SetTaskPushNotificationConfigRequest` object specifying the task ID and configuration.
            http_kwargs: Optional dictionary of keyword arguments to pass to the
                underlying httpx.post request.
            context: The client call context.

        Returns:
            A `SetTaskPushNotificationConfigResponse` object containing the confirmation or an error.

        Raises:
            A2AClientHTTPError: If an HTTP error occurs during the request.
            A2AClientJSONError: If the response body cannot be decoded as JSON or validated.
        """
        if not context and http_kwargs:
            context = ClientCallContext(state={'http_kwargs': http_kwargs})
        try:
            result = await self._transport.set_task_callback(
                request.params, context=context
            )
            return SetTaskPushNotificationConfigResponse(
                root=SetTaskPushNotificationConfigSuccessResponse(
                    id=request.id, jsonrpc='2.0', result=result
                )
            )
        except A2AClientJSONRPCError as e:
            return SetTaskPushNotificationConfigResponse(
                JSONRPCErrorResponse(error=e.error)
            )

    async def get_task_callback(
        self,
        request: GetTaskPushNotificationConfigRequest,
        *,
        http_kwargs: dict[str, Any] | None = None,
        context: ClientCallContext | None = None,
    ) -> GetTaskPushNotificationConfigResponse:
        """Retrieves the push notification configuration for a specific task.

        Args:
            request: The `GetTaskPushNotificationConfigRequest` object specifying the task ID.
            http_kwargs: Optional dictionary of keyword arguments to pass to the
                underlying httpx.post request.
            context: The client call context.

        Returns:
            A `GetTaskPushNotificationConfigResponse` object containing the configuration or an error.

        Raises:
            A2AClientHTTPError: If an HTTP error occurs during the request.
            A2AClientJSONError: If the response body cannot be decoded as JSON or validated.
        """
        if not context and http_kwargs:
            context = ClientCallContext(state={'http_kwargs': http_kwargs})
        params = request.params
        if isinstance(params, TaskIdParams):
            params = GetTaskPushNotificationConfigParams(id=request.params.id)
        try:
            result = await self._transport.get_task_callback(
                params, context=context
            )
            return GetTaskPushNotificationConfigResponse(
                root=GetTaskPushNotificationConfigSuccessResponse(
                    id=request.id, jsonrpc='2.0', result=result
                )
            )
        except A2AClientJSONRPCError as e:
            return GetTaskPushNotificationConfigResponse(
                JSONRPCErrorResponse(error=e.error)
            )

    async def resubscribe(
        self,
        request: TaskResubscriptionRequest,
        *,
        http_kwargs: dict[str, Any] | None = None,
        context: ClientCallContext | None = None,
    ) -> AsyncGenerator[SendStreamingMessageResponse, None]:
        """Reconnects to get task updates.

        This method uses Server-Sent Events (SSE) to receive a stream of updates from the agent.

        Args:
            request: The `TaskResubscriptionRequest` object containing the task information to reconnect to.
            http_kwargs: Optional dictionary of keyword arguments to pass to the
                underlying httpx.post request. A default `timeout=None` is set but can be overridden.
            context: The client call context.

        Yields:
            `SendStreamingMessageResponse` objects as they are received in the SSE stream.
            These can be Task, Message, TaskStatusUpdateEvent, or TaskArtifactUpdateEvent.

        Raises:
            A2AClientHTTPError: If an HTTP or SSE protocol error occurs during the request.
            A2AClientJSONError: If an SSE event data cannot be decoded as JSON or validated.
        """
        if not context and http_kwargs:
            context = ClientCallContext(state={'http_kwargs': http_kwargs})

        async for result in self._transport.resubscribe(
            request.params, context=context
        ):
            yield SendStreamingMessageResponse(
                root=SendStreamingMessageSuccessResponse(
                    id=request.id, jsonrpc='2.0', result=result
                )
            )

    async def get_card(
        self,
        *,
        http_kwargs: dict[str, Any] | None = None,
        context: ClientCallContext | None = None,
    ) -> AgentCard:
        """Retrieves the authenticated card (if necessary) or the public one.

        Args:
            http_kwargs: Optional dictionary of keyword arguments to pass to the
                underlying httpx.post request.
            context: The client call context.

        Returns:
            A `AgentCard` object containing the card or an error.

        Raises:
            A2AClientHTTPError: If an HTTP error occurs during the request.
            A2AClientJSONError: If the response body cannot be decoded as JSON or validated.
        """
        if not context and http_kwargs:
            context = ClientCallContext(state={'http_kwargs': http_kwargs})
        return await self._transport.get_card(context=context)



import asyncio
import logging

from collections.abc import AsyncGenerator
from typing import cast

from a2a.server.agent_execution import (
    AgentExecutor,
    RequestContext,
    RequestContextBuilder,
    SimpleRequestContextBuilder,
)
from a2a.server.context import ServerCallContext
from a2a.server.events import (
    Event,
    EventConsumer,
    EventQueue,
    InMemoryQueueManager,
    QueueManager,
)
from a2a.server.request_handlers.request_handler import RequestHandler
from a2a.server.tasks import (
    PushNotificationConfigStore,
    PushNotificationSender,
    ResultAggregator,
    TaskManager,
    TaskStore,
)
from a2a.types import (
    DeleteTaskPushNotificationConfigParams,
    GetTaskPushNotificationConfigParams,
    InternalError,
    InvalidParamsError,
    ListTaskPushNotificationConfigParams,
    Message,
    MessageSendParams,
    Task,
    TaskIdParams,
    TaskNotCancelableError,
    TaskNotFoundError,
    TaskPushNotificationConfig,
    TaskQueryParams,
    TaskState,
    UnsupportedOperationError,
)
from a2a.utils.errors import ServerError
from a2a.utils.task import apply_history_length
from a2a.utils.telemetry import SpanKind, trace_class


logger = logging.getLogger(__name__)

TERMINAL_TASK_STATES = {
    TaskState.completed,
    TaskState.canceled,
    TaskState.failed,
    TaskState.rejected,
}


@trace_class(kind=SpanKind.SERVER)
class DefaultRequestHandler(RequestHandler):
    """Default request handler for all incoming requests.

    This handler provides default implementations for all A2A JSON-RPC methods,
    coordinating between the `AgentExecutor`, `TaskStore`, `QueueManager`,
    and optional `PushNotifier`.
    """

    _running_agents: dict[str, asyncio.Task]
    _background_tasks: set[asyncio.Task]

    def __init__(  # noqa: PLR0913
        self,
        agent_executor: AgentExecutor,
        task_store: TaskStore,
        queue_manager: QueueManager | None = None,
        push_config_store: PushNotificationConfigStore | None = None,
        push_sender: PushNotificationSender | None = None,
        request_context_builder: RequestContextBuilder | None = None,
    ) -> None:
        """Initializes the DefaultRequestHandler.

        Args:
            agent_executor: The `AgentExecutor` instance to run agent logic.
            task_store: The `TaskStore` instance to manage task persistence.
            queue_manager: The `QueueManager` instance to manage event queues. Defaults to `InMemoryQueueManager`.
            push_config_store: The `PushNotificationConfigStore` instance for managing push notification configurations. Defaults to None.
            push_sender: The `PushNotificationSender` instance for sending push notifications. Defaults to None.
            request_context_builder: The `RequestContextBuilder` instance used
              to build request contexts. Defaults to `SimpleRequestContextBuilder`.
        """
        self.agent_executor = agent_executor
        self.task_store = task_store
        self._queue_manager = queue_manager or InMemoryQueueManager()
        self._push_config_store = push_config_store
        self._push_sender = push_sender
        self._request_context_builder = (
            request_context_builder
            or SimpleRequestContextBuilder(
                should_populate_referred_tasks=False, task_store=self.task_store
            )
        )
        # TODO: Likely want an interface for managing this, like AgentExecutionManager.
        self._running_agents = {}
        self._running_agents_lock = asyncio.Lock()
        # Tracks background tasks (e.g., deferred cleanups) to avoid orphaning
        # asyncio tasks and to surface unexpected exceptions.
        self._background_tasks = set()

    async def on_get_task(
        self,
        params: TaskQueryParams,
        context: ServerCallContext | None = None,
    ) -> Task | None:
        """Default handler for 'tasks/get'."""
        task: Task | None = await self.task_store.get(params.id, context)
        if not task:
            raise ServerError(error=TaskNotFoundError())

        # Apply historyLength parameter if specified
        return apply_history_length(task, params.history_length)

    async def on_cancel_task(
        self, params: TaskIdParams, context: ServerCallContext | None = None
    ) -> Task | None:
        """Default handler for 'tasks/cancel'.

        Attempts to cancel the task managed by the `AgentExecutor`.
        """
        task: Task | None = await self.task_store.get(params.id, context)
        if not task:
            raise ServerError(error=TaskNotFoundError())

        # Check if task is in a non-cancelable state (completed, canceled, failed, rejected)
        if task.status.state in TERMINAL_TASK_STATES:
            raise ServerError(
                error=TaskNotCancelableError(
                    message=f'Task cannot be canceled - current state: {task.status.state}'
                )
            )

        task_manager = TaskManager(
            task_id=task.id,
            context_id=task.context_id,
            task_store=self.task_store,
            initial_message=None,
            context=context,
        )
        result_aggregator = ResultAggregator(task_manager)

        queue = await self._queue_manager.tap(task.id)
        if not queue:
            queue = EventQueue()

        await self.agent_executor.cancel(
            RequestContext(
                None,
                task_id=task.id,
                context_id=task.context_id,
                task=task,
            ),
            queue,
        )
        # Cancel the ongoing task, if one exists.
        if producer_task := self._running_agents.get(task.id):
            producer_task.cancel()

        consumer = EventConsumer(queue)
        result = await result_aggregator.consume_all(consumer)
        if not isinstance(result, Task):
            raise ServerError(
                error=InternalError(
                    message='Agent did not return valid response for cancel'
                )
            )

        if result.status.state != TaskState.canceled:
            raise ServerError(
                error=TaskNotCancelableError(
                    message=f'Task cannot be canceled - current state: {result.status.state}'
                )
            )

        return result

    async def _run_event_stream(
        self, request: RequestContext, queue: EventQueue
    ) -> None:
        """Runs the agent's `execute` method and closes the queue afterwards.

        Args:
            request: The request context for the agent.
            queue: The event queue for the agent to publish to.
        """
        await self.agent_executor.execute(request, queue)
        await queue.close()

    async def _setup_message_execution(
        self,
        params: MessageSendParams,
        context: ServerCallContext | None = None,
    ) -> tuple[TaskManager, str, EventQueue, ResultAggregator, asyncio.Task]:
        """Common setup logic for both streaming and non-streaming message handling.

        Returns:
            A tuple of (task_manager, task_id, queue, result_aggregator, producer_task)
        """
        # Create task manager and validate existing task
        task_manager = TaskManager(
            task_id=params.message.task_id,
            context_id=params.message.context_id,
            task_store=self.task_store,
            initial_message=params.message,
            context=context,
        )
        task: Task | None = await task_manager.get_task()

        if task:
            if task.status.state in TERMINAL_TASK_STATES:
                raise ServerError(
                    error=InvalidParamsError(
                        message=f'Task {task.id} is in terminal state: {task.status.state.value}'
                    )
                )

            task = task_manager.update_with_message(params.message, task)
        elif params.message.task_id:
            raise ServerError(
                error=TaskNotFoundError(
                    message=f'Task {params.message.task_id} was specified but does not exist'
                )
            )

        # Build request context
        request_context = await self._request_context_builder.build(
            params=params,
            task_id=task.id if task else None,
            context_id=params.message.context_id,
            task=task,
            context=context,
        )

        task_id = cast('str', request_context.task_id)
        # Always assign a task ID. We may not actually upgrade to a task, but
        # dictating the task ID at this layer is useful for tracking running
        # agents.

        if (
            self._push_config_store
            and params.configuration
            and params.configuration.push_notification_config
        ):
            await self._push_config_store.set_info(
                task_id, params.configuration.push_notification_config
            )

        queue = await self._queue_manager.create_or_tap(task_id)
        result_aggregator = ResultAggregator(task_manager)
        # TODO: to manage the non-blocking flows.
        producer_task = asyncio.create_task(
            self._run_event_stream(request_context, queue)
        )
        await self._register_producer(task_id, producer_task)

        return task_manager, task_id, queue, result_aggregator, producer_task

    def _validate_task_id_match(self, task_id: str, event_task_id: str) -> None:
        """Validates that agent-generated task ID matches the expected task ID."""
        if task_id != event_task_id:
            logger.error(
                'Agent generated task_id=%s does not match the RequestContext task_id=%s.',
                event_task_id,
                task_id,
            )
            raise ServerError(
                InternalError(message='Task ID mismatch in agent response')
            )

    async def _send_push_notification_if_needed(
        self, task_id: str, result_aggregator: ResultAggregator
    ) -> None:
        """Sends push notification if configured and task is available."""
        if self._push_sender and task_id:
            latest_task = await result_aggregator.current_result
            if isinstance(latest_task, Task):
                await self._push_sender.send_notification(latest_task)

    async def on_message_send(
        self,
        params: MessageSendParams,
        context: ServerCallContext | None = None,
    ) -> Message | Task:
        """Default handler for 'message/send' interface (non-streaming).

        Starts the agent execution for the message and waits for the final
        result (Task or Message).
        """
        (
            _task_manager,
            task_id,
            queue,
            result_aggregator,
            producer_task,
        ) = await self._setup_message_execution(params, context)

        consumer = EventConsumer(queue)
        producer_task.add_done_callback(consumer.agent_task_callback)

        blocking = True  # Default to blocking behavior
        if params.configuration and params.configuration.blocking is False:
            blocking = False

        interrupted_or_non_blocking = False
        try:
            # Create async callback for push notifications
            async def push_notification_callback() -> None:
                await self._send_push_notification_if_needed(
                    task_id, result_aggregator
                )

            (
                result,
                interrupted_or_non_blocking,
            ) = await result_aggregator.consume_and_break_on_interrupt(
                consumer,
                blocking=blocking,
                event_callback=push_notification_callback,
            )

        except Exception:
            logger.exception('Agent execution failed')
            raise
        finally:
            if interrupted_or_non_blocking:
                cleanup_task = asyncio.create_task(
                    self._cleanup_producer(producer_task, task_id)
                )
                cleanup_task.set_name(f'cleanup_producer:{task_id}')
                self._track_background_task(cleanup_task)
            else:
                await self._cleanup_producer(producer_task, task_id)

        if not result:
            raise ServerError(error=InternalError())

        if isinstance(result, Task):
            self._validate_task_id_match(task_id, result.id)
            if params.configuration:
                result = apply_history_length(
                    result, params.configuration.history_length
                )

        await self._send_push_notification_if_needed(task_id, result_aggregator)

        return result

    async def on_message_send_stream(
        self,
        params: MessageSendParams,
        context: ServerCallContext | None = None,
    ) -> AsyncGenerator[Event]:
        """Default handler for 'message/stream' (streaming).

        Starts the agent execution and yields events as they are produced
        by the agent.
        """
        (
            _task_manager,
            task_id,
            queue,
            result_aggregator,
            producer_task,
        ) = await self._setup_message_execution(params, context)
        consumer = EventConsumer(queue)
        producer_task.add_done_callback(consumer.agent_task_callback)

        try:
            async for event in result_aggregator.consume_and_emit(consumer):
                if isinstance(event, Task):
                    self._validate_task_id_match(task_id, event.id)

                await self._send_push_notification_if_needed(
                    task_id, result_aggregator
                )
                yield event
        except (asyncio.CancelledError, GeneratorExit):
            # Client disconnected: continue consuming and persisting events in the background
            bg_task = asyncio.create_task(
                result_aggregator.consume_all(consumer)
            )
            bg_task.set_name(f'background_consume:{task_id}')
            self._track_background_task(bg_task)
            raise
        finally:
            cleanup_task = asyncio.create_task(
                self._cleanup_producer(producer_task, task_id)
            )
            cleanup_task.set_name(f'cleanup_producer:{task_id}')
            self._track_background_task(cleanup_task)

    async def _register_producer(
        self, task_id: str, producer_task: asyncio.Task
    ) -> None:
        """Registers the agent execution task with the handler."""
        async with self._running_agents_lock:
            self._running_agents[task_id] = producer_task

    def _track_background_task(self, task: asyncio.Task) -> None:
        """Tracks a background task and logs exceptions on completion.

        This avoids unreferenced tasks (and associated lint warnings) while
        ensuring any exceptions are surfaced in logs.
        """
        self._background_tasks.add(task)

        def _on_done(completed: asyncio.Task) -> None:
            try:
                # Retrieve result to raise exceptions, if any
                completed.result()
            except asyncio.CancelledError:
                name = completed.get_name()
                logger.debug('Background task %s cancelled', name)
            except Exception:
                name = completed.get_name()
                logger.exception('Background task %s failed', name)
            finally:
                self._background_tasks.discard(completed)

        task.add_done_callback(_on_done)

    async def _cleanup_producer(
        self,
        producer_task: asyncio.Task,
        task_id: str,
    ) -> None:
        """Cleans up the agent execution task and queue manager entry."""
        await producer_task
        await self._queue_manager.close(task_id)
        async with self._running_agents_lock:
            self._running_agents.pop(task_id, None)

    async def on_set_task_push_notification_config(
        self,
        params: TaskPushNotificationConfig,
        context: ServerCallContext | None = None,
    ) -> TaskPushNotificationConfig:
        """Default handler for 'tasks/pushNotificationConfig/set'.

        Requires a `PushNotifier` to be configured.
        """
        if not self._push_config_store:
            raise ServerError(error=UnsupportedOperationError())

        task: Task | None = await self.task_store.get(params.task_id, context)
        if not task:
            raise ServerError(error=TaskNotFoundError())

        await self._push_config_store.set_info(
            params.task_id,
            params.push_notification_config,
        )

        return params

    async def on_get_task_push_notification_config(
        self,
        params: TaskIdParams | GetTaskPushNotificationConfigParams,
        context: ServerCallContext | None = None,
    ) -> TaskPushNotificationConfig:
        """Default handler for 'tasks/pushNotificationConfig/get'.

        Requires a `PushConfigStore` to be configured.
        """
        if not self._push_config_store:
            raise ServerError(error=UnsupportedOperationError())

        task: Task | None = await self.task_store.get(params.id, context)
        if not task:
            raise ServerError(error=TaskNotFoundError())

        push_notification_config = await self._push_config_store.get_info(
            params.id
        )
        if not push_notification_config or not push_notification_config[0]:
            raise ServerError(
                error=InternalError(
                    message='Push notification config not found'
                )
            )

        return TaskPushNotificationConfig(
            task_id=params.id,
            push_notification_config=push_notification_config[0],
        )

    async def on_resubscribe_to_task(
        self,
        params: TaskIdParams,
        context: ServerCallContext | None = None,
    ) -> AsyncGenerator[Event]:
        """Default handler for 'tasks/resubscribe'.

        Allows a client to re-attach to a running streaming task's event stream.
        Requires the task and its queue to still be active.
        """
        task: Task | None = await self.task_store.get(params.id, context)
        if not task:
            raise ServerError(error=TaskNotFoundError())

        if task.status.state in TERMINAL_TASK_STATES:
            raise ServerError(
                error=InvalidParamsError(
                    message=f'Task {task.id} is in terminal state: {task.status.state.value}'
                )
            )

        task_manager = TaskManager(
            task_id=task.id,
            context_id=task.context_id,
            task_store=self.task_store,
            initial_message=None,
            context=context,
        )

        result_aggregator = ResultAggregator(task_manager)

        queue = await self._queue_manager.tap(task.id)
        if not queue:
            raise ServerError(error=TaskNotFoundError())

        consumer = EventConsumer(queue)
        async for event in result_aggregator.consume_and_emit(consumer):
            yield event

    async def on_list_task_push_notification_config(
        self,
        params: ListTaskPushNotificationConfigParams,
        context: ServerCallContext | None = None,
    ) -> list[TaskPushNotificationConfig]:
        """Default handler for 'tasks/pushNotificationConfig/list'.

        Requires a `PushConfigStore` to be configured.
        """
        if not self._push_config_store:
            raise ServerError(error=UnsupportedOperationError())

        task: Task | None = await self.task_store.get(params.id, context)
        if not task:
            raise ServerError(error=TaskNotFoundError())

        push_notification_config_list = await self._push_config_store.get_info(
            params.id
        )

        return [
            TaskPushNotificationConfig(
                task_id=params.id, push_notification_config=config
            )
            for config in push_notification_config_list
        ]

    async def on_delete_task_push_notification_config(
        self,
        params: DeleteTaskPushNotificationConfigParams,
        context: ServerCallContext | None = None,
    ) -> None:
        """Default handler for 'tasks/pushNotificationConfig/delete'.

        Requires a `PushConfigStore` to be configured.
        """
        if not self._push_config_store:
            raise ServerError(error=UnsupportedOperationError())

        task: Task | None = await self.task_store.get(params.id, context)
        if not task:
            raise ServerError(error=TaskNotFoundError())

        await self._push_config_store.delete_info(
            params.id, params.push_notification_config_id
        )



import unittest
from unittest.mock import Mock, patch
import uuid
import requests
from pygeai.proxy.clients import (
    ProxyClient, ToolProxyData, ToolProxyJob, 
    ToolProxyJobResult
)
from pygeai.proxy.tool import ProxiedTool


class TestToolProxyData(unittest.TestCase):
    """
    python -m unittest pygeai.tests.proxy.test_clients.TestToolProxyData
    """

    def setUp(self):
        """Set up test fixtures."""
        self.test_uuid = uuid.uuid4()
        self.test_affinity = uuid.uuid4()
        self.tool = ProxiedTool(
            server_name="test_server",
            name="test_tool",
            description="Test tool",
            public_prefix="public.prefix",
            input_schema={"type": "object"}
        )

    def test_initialization_with_defaults(self):
        """Test initialization with default values."""
        data = ToolProxyData()
        self.assertIsNone(data.id)
        self.assertIsNone(data.name)
        self.assertIsNone(data.description)
        self.assertIsNone(data.affinity)
        self.assertIsNone(data.tools)

    def test_initialization_with_values(self):
        """Test initialization with provided values."""
        data = ToolProxyData(
            id=self.test_uuid,
            name="Test Proxy",
            description="Test Description",
            affinity=self.test_affinity,
            tools=[self.tool]
        )
        self.assertEqual(data.id, self.test_uuid)
        self.assertEqual(data.name, "Test Proxy")
        self.assertEqual(data.description, "Test Description")
        self.assertEqual(data.affinity, self.test_affinity)
        self.assertEqual(data.tools, [self.tool])

    def test_to_dict_with_all_values(self):
        """Test converting to dictionary with all values."""
        data = ToolProxyData(
            id=self.test_uuid,
            name="Test Proxy",
            description="Test Description",
            affinity=self.test_affinity,
            tools=[self.tool]
        )
        result = data.to_dict()
        
        expected = {
            "id": str(self.test_uuid),
            "name": "Test Proxy",
            "description": "Test Description",
            "affinity": str(self.test_affinity),
            "tools": [
                {
                    "name": self.tool.get_full_name(),
                    "publicName": self.tool.get_public_name(),
                    "server": self.tool.server_name,
                    "description": self.tool.description,
                    "inputSchema": self.tool.format_for_llm()
                }
            ]
        }
        self.assertEqual(result, expected)

    def test_to_dict_with_none_values(self):
        """Test converting to dictionary with None values."""
        data = ToolProxyData()
        result = data.to_dict()
        
        expected = {
            "id": None,
            "name": None,
            "description": None,
            "affinity": str(uuid.UUID(int=0)),
            "tools": []
        }
        self.assertEqual(result, expected)

    def test_to_dict_with_private_tool(self):
        """Test converting to dictionary with private tool."""
        private_tool = ProxiedTool(
            server_name="server",
            name="tool",
            description="desc",
            public_prefix=None,
            input_schema={}
        )
        data = ToolProxyData(tools=[private_tool])
        result = data.to_dict()
        
        tool_dict = result["tools"][0]
        self.assertNotIn("publicName", tool_dict)


class TestToolProxyJob(unittest.TestCase):
    """
    python -m unittest pygeai.tests.proxy.test_clients.TestToolProxyJob
    """

    def setUp(self):
        """Set up test fixtures."""
        self.test_uuid = uuid.uuid4()
        self.proxy_uuid = uuid.uuid4()

    def test_initialization(self):
        """Test job initialization."""
        job = ToolProxyJob(
            id=self.test_uuid,
            proxy_id=self.proxy_uuid,
            proxy_status="active",
            job_status="pending",
            input="test input",
            output="test output",
            server="test_server"
        )
        self.assertEqual(job.id, self.test_uuid)
        self.assertEqual(job.proxy_id, self.proxy_uuid)
        self.assertEqual(job.proxy_status, "active")
        self.assertEqual(job.job_status, "pending")
        self.assertEqual(job.input, "test input")
        self.assertEqual(job.output, "test output")
        self.assertEqual(job.server, "test_server")

    def test_from_dict(self):
        """Test creating job from dictionary."""
        data = {
            "id": str(self.test_uuid),
            "proxyId": str(self.proxy_uuid),
            "proxyStatus": "active",
            "jobStatus": "pending",
            "input": "test input",
            "output": "test output",
            "server": "test_server"
        }
        job = ToolProxyJob.from_dict(data)
        
        self.assertEqual(job.id, self.test_uuid)
        self.assertEqual(job.proxy_id, self.proxy_uuid)
        self.assertEqual(job.proxy_status, "active")
        self.assertEqual(job.job_status, "pending")
        self.assertEqual(job.input, "test input")
        self.assertEqual(job.output, "test output")
        self.assertEqual(job.server, "test_server")

    def test_from_dict_with_optional_fields(self):
        """Test creating job from dictionary with optional fields."""
        data = {
            "id": str(self.test_uuid),
            "proxyId": str(self.proxy_uuid),
            "proxyStatus": "active",
            "jobStatus": "pending"
        }
        job = ToolProxyJob.from_dict(data)
        
        self.assertEqual(job.id, self.test_uuid)
        self.assertEqual(job.proxy_id, self.proxy_uuid)
        self.assertEqual(job.proxy_status, "active")
        self.assertEqual(job.job_status, "pending")
        self.assertIsNone(job.input)
        self.assertIsNone(job.output)
        self.assertIsNone(job.server)


class TestToolProxyJobResult(unittest.TestCase):
    """
    python -m unittest pygeai.tests.proxy.test_clients.TestToolProxyJobResult
    """

    def setUp(self):
        """Set up test fixtures."""
        self.test_uuid = uuid.uuid4()
        self.proxy_uuid = uuid.uuid4()
        self.job = ToolProxyJob(
            id=self.test_uuid,
            proxy_id=self.proxy_uuid,
            proxy_status="active",
            job_status="completed",
            input="test input",
            output="test output",
            server="test_server"
        )

    def test_initialization(self):
        """Test result initialization."""
        result = ToolProxyJobResult(success=True, job=self.job)
        self.assertTrue(result.success)
        self.assertEqual(result.job, self.job)

    def test_to_dict(self):
        """Test converting result to dictionary."""
        result = ToolProxyJobResult(success=True, job=self.job)
        result_dict = result.to_dict()
        
        expected = {
            "success": True,
            "job": {
                "id": str(self.test_uuid),
                "jobStatus": "completed",
                "output": "test output"
            }
        }
        self.assertEqual(result_dict, expected)


class TestProxyClient(unittest.TestCase):
    """
    python -m unittest pygeai.tests.proxy.test_clients.TestProxyClient
    """

    def setUp(self):
        """Set up test fixtures."""
        self.api_key = "test_api_key"
        self.base_url = "https://api.example.com"
        self.proxy_id = uuid.uuid4()
        self.client = ProxyClient(self.api_key, self.base_url, self.proxy_id)

    def test_initialization(self):
        """Test client initialization."""
        self.assertEqual(self.client.proxy_id, self.proxy_id)
        self.assertEqual(self.client.base_url, "https://api.example.com/v2/tool-proxy")
        self.assertEqual(self.client.session.headers["Authorization"], f"Bearer {self.api_key}")
        self.assertEqual(self.client.session.timeout, (5, 30))

    def test_initialization_with_trailing_slash(self):
        """Test client initialization with trailing slash in base URL."""
        client = ProxyClient(self.api_key, "https://api.example.com/", self.proxy_id)
        self.assertEqual(client.base_url, "https://api.example.com/v2/tool-proxy")

    @patch('requests.Session')
    def test_make_request_success(self, mock_session):
        """Test successful request."""
        mock_response = Mock()
        mock_response.json.return_value = {"status": "success"}
        mock_response.status_code = 200
        mock_response.raise_for_status.return_value = None
        mock_session.return_value.request.return_value = mock_response
        
        client = ProxyClient(self.api_key, self.base_url, self.proxy_id)
        result = client._make_request("GET", "/test")
        
        self.assertEqual(result, {"status": "success"})

    @patch('requests.Session')
    def test_make_request_timeout(self, mock_session):
        """Test request timeout."""
        mock_session.return_value.request.side_effect = requests.exceptions.Timeout()
        
        client = ProxyClient(self.api_key, self.base_url, self.proxy_id)
        
        with self.assertRaises(TimeoutError):
            client._make_request("GET", "/test")

    @patch('requests.Session')
    def test_make_request_http_error(self, mock_session):
        """Test HTTP error."""
        mock_session.return_value.request.side_effect = requests.exceptions.HTTPError("404 Not Found")
        
        client = ProxyClient(self.api_key, self.base_url, self.proxy_id)
        
        with self.assertRaises(requests.exceptions.HTTPError):
            client._make_request("GET", "/test")

    @patch('requests.Session')
    def test_make_request_connection_error(self, mock_session):
        """Test connection error."""
        mock_session.return_value.request.side_effect = requests.exceptions.ConnectionError()
        
        client = ProxyClient(self.api_key, self.base_url, self.proxy_id)
        
        with self.assertRaises(ConnectionError):
            client._make_request("GET", "/test")

    @patch('requests.Session')
    def test_make_request_ssl_error(self, mock_session):
        """Test SSL error."""
        mock_session.return_value.request.side_effect = requests.exceptions.SSLError()
        
        client = ProxyClient(self.api_key, self.base_url, self.proxy_id)
        
        with self.assertRaises(requests.exceptions.SSLError):
            client._make_request("GET", "/test")

    @patch('requests.Session')
    def test_make_request_proxy_error(self, mock_session):
        """Test proxy error."""
        mock_session.return_value.request.side_effect = requests.exceptions.ProxyError()
        
        client = ProxyClient(self.api_key, self.base_url, self.proxy_id)
        
        with self.assertRaises(requests.exceptions.ProxyError):
            client._make_request("GET", "/test")

    @patch('requests.Session')
    def test_make_request_json_decode_error(self, mock_session):
        """Test JSON decode error."""
        mock_response = Mock()
        mock_response.json.side_effect = ValueError("Invalid JSON")
        mock_response.text = "plain text response"
        mock_response.status_code = 200
        mock_response.raise_for_status.return_value = None
        mock_session.return_value.request.return_value = mock_response
        
        client = ProxyClient(self.api_key, self.base_url, self.proxy_id)
        result = client._make_request("GET", "/test")
        
        self.assertEqual(result, "plain text response")

    @patch.object(ProxyClient, '_make_request')
    def test_register(self, mock_make_request):
        """Test proxy registration."""
        mock_make_request.return_value = {"status": "registered"}
        
        tool = ProxiedTool(
            server_name="test_server",
            name="test_tool",
            description="Test tool",
            public_prefix="public.prefix",
            input_schema={"type": "object"}
        )
        
        proxy_data = ToolProxyData(
            name="Test Proxy",
            description="Test Description",
            tools=[tool]
        )
        
        result = self.client.register(proxy_data)
        
        self.assertEqual(result, {"status": "registered"})
        self.assertEqual(proxy_data.id, self.proxy_id)
        mock_make_request.assert_called_once_with(
            'POST', '/register', 
            json={"server": proxy_data.to_dict()}
        )

    @patch.object(ProxyClient, '_make_request')
    def test_dequeue(self, mock_make_request):
        """Test job dequeuing."""
        mock_jobs = [
            {
                "id": str(uuid.uuid4()),
                "proxyId": str(self.proxy_id),
                "proxyStatus": "active",
                "jobStatus": "pending",
                "input": "test input",
                "output": None,
                "server": "test_server"
            }
        ]
        mock_make_request.return_value = mock_jobs
        
        result = self.client.dequeue()
        
        self.assertEqual(len(result), 1)
        self.assertIsInstance(result[0], ToolProxyJob)
        mock_make_request.assert_called_once_with('GET', f'/dequeue/{self.proxy_id}')

    @patch.object(ProxyClient, '_make_request')
    def test_send_result(self, mock_make_request):
        """Test sending job result."""
        job = ToolProxyJob(
            id=uuid.uuid4(),
            proxy_id=self.proxy_id,
            proxy_status="active",
            job_status="completed",
            input="test input",
            output="test output",
            server="test_server"
        )
        result = ToolProxyJobResult(success=True, job=job)
        
        self.client.send_result(result)
        
        mock_make_request.assert_called_once_with(
            'POST', f'/result/{self.proxy_id}',
            json={"jobResult": result.to_dict()}
        )


if __name__ == '__main__':
    unittest.main() 


import unittest
from unittest.mock import patch, MagicMock
from pygeai.cli.commands.lab.ai_lab import (
    show_help,
    list_agents,
    create_agent,
    get_agent,
    create_sharing_link,
    publish_agent_revision,
    delete_agent,
    update_agent,
    create_tool,
    list_tools,
    get_tool,
    delete_tool,
    update_tool,
    publish_tool_revision,
    get_parameter,
    set_parameter,
    list_reasoning_strategies,
    create_reasoning_strategy,
    update_reasoning_strategy,
    get_reasoning_strategy,
    create_process,
    update_process,
    get_process,
    list_processes,
    list_processes_instances,
    delete_process,
    publish_process_revision,
    create_task,
    get_task,
    list_tasks,
    update_task,
    delete_task,
    publish_task_revision,
    start_instance,
    abort_instance,
    get_instance,
    get_thread_information,
    send_user_signal,
    create_kb,
    get_kb,
    list_kbs,
    delete_kb,
    list_jobs
)
from pygeai.core.common.exceptions import MissingRequirementException, WrongArgumentError
from pygeai.cli.commands import Option


class TestAILab(unittest.TestCase):
    """
    python -m unittest pygeai.tests.cli.commands.lab.test_ai_lab.TestAILab
    """

    def test_show_help(self):
        with patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            show_help()
            mock_stdout.assert_called_once()

    def test_list_agents_auto_fetch_project_id(self):
        option_list = []
        with patch('pygeai.cli.commands.lab.ai_lab.AgentClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client_instance.project_id = "auto_fetched_proj"
            mock_client.return_value = mock_client_instance
            mock_client_instance.list_agents.return_value = []
            list_agents(option_list)
            mock_client.assert_called_once_with(project_id=None)
            mock_stdout.assert_called_once_with("Agent list: \n[]")

    def test_list_agents_success(self):
        option_list = [(Option("project_id", ["--project-id"], "", True), "proj123")]
        with patch('pygeai.cli.commands.lab.ai_lab.AgentClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.list_agents.return_value = [{"id": "agent1"}]
            list_agents(option_list)
            mock_client_instance.list_agents.assert_called_once_with(
                status="", start="", count="", access_scope="public",
                allow_drafts=True, allow_external=False
            )
            mock_stdout.assert_called_once_with("Agent list: \n[{'id': 'agent1'}]")

    def test_create_agent_missing_required_parameters(self):
        option_list = [(Option("project_id", ["--project-id"], "", True), "proj123")]
        with self.assertRaises(MissingRequirementException) as cm:
            create_agent(option_list)
        self.assertEqual(str(cm.exception), "Cannot create assistant without specifying name.")

    def test_create_agent_invalid_input_json(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("name", ["--name"], "", True), "agent1"),
            (Option("access_scope", ["--access-scope"], "", True), "public"),
            (Option("public_name", ["--public-name"], "", True), "pub.agent1"),
            (Option("agent_data_prompt_input", ["--agent-data-prompt-input"], "", True), "invalid_json[")
        ]
        with self.assertRaises(WrongArgumentError) as cm:
            create_agent(option_list)
        self.assertIn("Inputs must be a list of strings", str(cm.exception))

    def test_create_agent_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("name", ["--name"], "", True), "agent1"),
            (Option("access_scope", ["--access-scope"], "", True), "public"),
            (Option("public_name", ["--public-name"], "", True), "pub.agent1"),
            (Option("agent_data_prompt_input", ["--agent-data-prompt-input"], "", True), '["input1"]'),
            (Option("agent_data_prompt_output", ["--agent-data-prompt-output"], "", True), '{"key": "out1", "description": "desc"}'),
            (Option("agent_data_prompt_example", ["--agent-data-prompt-example"], "", True), '{"inputData": "ex", "output": "out"}'),
            (Option("agent_data_resource_pools", ["--agent-data-resource-pools"], "", True), '[{"name": "pool1"}]')
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgentClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout, \
             patch('pygeai.cli.commands.lab.ai_lab.get_agent_data_prompt_inputs', return_value=["input1"]), \
             patch('pygeai.cli.commands.lab.ai_lab.get_agent_data_prompt_outputs', return_value=[{"key": "out1", "description": "desc"}]), \
             patch('pygeai.cli.commands.lab.ai_lab.get_agent_data_prompt_examples', return_value=[{"inputData": "ex", "output": "out"}]):
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.create_agent.return_value = {"id": "agent1"}
            create_agent(option_list)
            mock_client_instance.create_agent.assert_called_once()
            mock_stdout.assert_called_once_with("New agent detail: \n{'id': 'agent1'}")

    def test_get_agent_missing_required_parameters(self):
        option_list = []
        with self.assertRaises(MissingRequirementException) as cm:
            get_agent(option_list)
        self.assertEqual(str(cm.exception), "Agent ID must be specified.")

    def test_get_agent_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("agent_id", ["--agent-id"], "", True), "agent1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgentClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.get_agent.return_value = {"id": "agent1"}
            get_agent(option_list)
            mock_client_instance.get_agent.assert_called_once_with(
                agent_id="agent1", revision=0, version=0, allow_drafts=True
            )
            mock_stdout.assert_called_once_with("Agent detail: \n{'id': 'agent1'}")

    def test_create_sharing_link_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("agent_id", ["--agent-id"], "", True), "agent1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgentClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.create_sharing_link.return_value = {"token": "share123"}
            create_sharing_link(option_list)
            mock_client_instance.create_sharing_link.assert_called_once_with(
                agent_id="agent1"
            )
            mock_stdout.assert_called_once_with("Sharing token: \n{'token': 'share123'}")

    def test_publish_agent_revision_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("agent_id", ["--agent-id"], "", True), "agent1"),
            (Option("revision", ["--revision"], "", True), "1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgentClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.publish_agent_revision.return_value = {"status": "published"}
            publish_agent_revision(option_list)
            mock_client_instance.publish_agent_revision.assert_called_once_with(
                agent_id="agent1", revision="1"
            )
            mock_stdout.assert_called_once_with("Published revision detail: \n{'status': 'published'}")

    def test_delete_agent_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("agent_id", ["--agent-id"], "", True), "agent1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgentClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.delete_agent.return_value = {"status": "deleted"}
            delete_agent(option_list)
            mock_client_instance.delete_agent.assert_called_once_with(
                agent_id="agent1"
            )
            mock_stdout.assert_called_once_with("Deleted agent detail: \n{'status': 'deleted'}")

    def test_update_agent_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("agent_id", ["--agent-id"], "", True), "agent1"),
            (Option("name", ["--name"], "", True), "agent_updated"),
            (Option("access_scope", ["--access-scope"], "", True), "public"),
            (Option("public_name", ["--public-name"], "", True), "pub.agent_updated")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgentClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout, \
             patch('pygeai.cli.commands.lab.ai_lab.get_agent_data_prompt_inputs', return_value=[]), \
             patch('pygeai.cli.commands.lab.ai_lab.get_agent_data_prompt_outputs', return_value=[]), \
             patch('pygeai.cli.commands.lab.ai_lab.get_agent_data_prompt_examples', return_value=[]):
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.update_agent.return_value = {"id": "agent1", "name": "agent_updated"}
            update_agent(option_list)
            mock_client_instance.update_agent.assert_called_once()
            mock_stdout.assert_called_once_with("Updated agent detail: \n{'id': 'agent1', 'name': 'agent_updated'}")

    # Tool-related tests
    def test_create_tool_missing_required_parameters(self):
        option_list = [(Option("project_id", ["--project-id"], "", True), "proj123")]
        with self.assertRaises(MissingRequirementException) as cm:
            create_tool(option_list)
        self.assertEqual(str(cm.exception), "Tool name must be specified.")

    def test_create_tool_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("name", ["--name"], "", True), "tool1"),
            (Option("access_scope", ["--access-scope"], "", True), "private")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.ToolClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout, \
             patch('pygeai.cli.commands.lab.ai_lab.get_tool_parameters', return_value=[]):
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.create_tool.return_value = {"id": "tool1"}
            create_tool(option_list)
            mock_client_instance.create_tool.assert_called_once()
            mock_stdout.assert_called_once_with("New tool detail: \n{'id': 'tool1'}")

    def test_list_tools_invalid_scope(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("scope", ["--scope"], "", True), "invalid")
        ]
        with self.assertRaises(ValueError) as cm:
            list_tools(option_list)
        self.assertIn("Scope must be one of", str(cm.exception))

    def test_list_tools_success(self):
        option_list = [(Option("project_id", ["--project-id"], "", True), "proj123")]
        with patch('pygeai.cli.commands.lab.ai_lab.ToolClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.list_tools.return_value = [{"id": "tool1"}]
            list_tools(option_list)
            mock_client_instance.list_tools.assert_called_once_with(
                id="", count="100", access_scope="public",
                allow_drafts=True, scope="api", allow_external=True
            )
            mock_stdout.assert_called_once_with("Tool list: \n[{'id': 'tool1'}]")

    def test_get_tool_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("tool_id", ["--tool-id"], "", True), "tool1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.ToolClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.get_tool.return_value = {"id": "tool1"}
            get_tool(option_list)
            mock_client_instance.get_tool.assert_called_once_with(
                tool_id="tool1", revision=0, version=0, allow_drafts=True
            )
            mock_stdout.assert_called_once_with("Tool detail: \n{'id': 'tool1'}")

    def test_delete_tool_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("tool_id", ["--tool-id"], "", True), "tool1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.ToolClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.delete_tool.return_value = {"status": "deleted"}
            delete_tool(option_list)
            mock_client_instance.delete_tool.assert_called_once_with(
                tool_id="tool1", tool_name=None
            )
            mock_stdout.assert_called_once_with("Deleted tool detail: \n{'status': 'deleted'}")

    def test_update_tool_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("tool_id", ["--tool-id"], "", True), "tool1"),
            (Option("name", ["--name"], "", True), "tool_updated")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.ToolClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout, \
             patch('pygeai.cli.commands.lab.ai_lab.get_tool_parameters', return_value=[]):
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.update_tool.return_value = {"id": "tool1", "name": "tool_updated"}
            update_tool(option_list)
            mock_client_instance.update_tool.assert_called_once()
            mock_stdout.assert_called_once_with("Updated tool detail: \n{'id': 'tool1', 'name': 'tool_updated'}")

    def test_publish_tool_revision_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("tool_id", ["--tool-id"], "", True), "tool1"),
            (Option("revision", ["--revision"], "", True), "1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.ToolClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.publish_tool_revision.return_value = {"status": "published"}
            publish_tool_revision(option_list)
            mock_client_instance.publish_tool_revision.assert_called_once_with(
                tool_id="tool1", revision="1"
            )
            mock_stdout.assert_called_once_with("Published revision detail: \n{'status': 'published'}")

    def test_get_parameter_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("tool_id", ["--tool-id"], "", True), "tool1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.ToolClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.get_parameter.return_value = {"key": "param1"}
            get_parameter(option_list)
            mock_client_instance.get_parameter.assert_called_once_with(
                tool_id="tool1", tool_public_name=None, revision=0, version=0, allow_drafts=True
            )
            mock_stdout.assert_called_once_with("Parameter detail: \n{'key': 'param1'}")

    def test_set_parameter_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("tool_id", ["--tool-id"], "", True), "tool1"),
            (Option("parameter", ["--parameter"], "", True), '{"key": "param1", "dataType": "String"}')
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.ToolClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout, \
             patch('pygeai.cli.commands.lab.ai_lab.get_tool_parameters', return_value=[{"key": "param1", "dataType": "String"}]):
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.set_parameter.return_value = {"status": "set"}
            set_parameter(option_list)
            mock_client_instance.set_parameter.assert_called_once_with(
                tool_id="tool1", tool_public_name=None, parameters=[{"key": "param1", "dataType": "String"}]
            )
            mock_stdout.assert_called_once_with("Set parameter detail: \n{'status': 'set'}")

    def test_list_reasoning_strategies_invalid_access_scope(self):
        option_list = [(Option("access_scope", ["--access-scope"], "", True), "invalid")]
        with self.assertRaises(WrongArgumentError) as cm:
            list_reasoning_strategies(option_list)
        self.assertEqual(str(cm.exception), "Access scope must be either 'public' or 'private'.")

    def test_list_reasoning_strategies_success(self):
        option_list = []
        with patch('pygeai.cli.commands.lab.ai_lab.ReasoningStrategyClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.list_reasoning_strategies.return_value = [{"id": "rs1"}]
            list_reasoning_strategies(option_list)
            mock_client_instance.list_reasoning_strategies.assert_called_once_with(
                name="", start="0", count="100", allow_external=True, access_scope="public"
            )
            mock_stdout.assert_called_once_with("Reasoning strategies list: \n[{'id': 'rs1'}]")

    def test_create_reasoning_strategy_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("name", ["--name"], "", True), "rs1"),
            (Option("system_prompt", ["--system-prompt"], "", True), "prompt text")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.ReasoningStrategyClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.create_reasoning_strategy.return_value = {"id": "rs1"}
            create_reasoning_strategy(option_list)
            mock_client_instance.create_reasoning_strategy.assert_called_once()
            mock_stdout.assert_called_once_with("Created reasoning strategy detail: \n{'id': 'rs1'}")

    def test_update_reasoning_strategy_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("reasoning_strategy_id", ["--reasoning-strategy-id"], "", True), "rs1"),
            (Option("name", ["--name"], "", True), "rs_updated")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.ReasoningStrategyClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.update_reasoning_strategy.return_value = {"id": "rs1", "name": "rs_updated"}
            update_reasoning_strategy(option_list)
            mock_client_instance.update_reasoning_strategy.assert_called_once()
            mock_stdout.assert_called_once_with("Updated reasoning strategy detail: \n{'id': 'rs1', 'name': 'rs_updated'}")

    def test_get_reasoning_strategy_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("reasoning_strategy_id", ["--reasoning-strategy-id"], "", True), "rs1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.ReasoningStrategyClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.get_reasoning_strategy.return_value = {"id": "rs1"}
            get_reasoning_strategy(option_list)
            mock_client_instance.get_reasoning_strategy.assert_called_once_with(
                reasoning_strategy_id="rs1", reasoning_strategy_name=None
            )
            mock_stdout.assert_called_once_with("Reasoning strategy detail: \n{'id': 'rs1'}")

    # Process-related tests
    def test_create_process_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("key", ["--key"], "", True), "proc_key"),
            (Option("name", ["--name"], "", True), "proc1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.create_process.return_value = {"id": "proc1"}
            create_process(option_list)
            mock_client_instance.create_process.assert_called_once()
            mock_stdout.assert_called_once_with("Created process detail: \n{'id': 'proc1'}")

    def test_update_process_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("process_id", ["--process-id"], "", True), "proc1"),
            (Option("name", ["--name"], "", True), "proc_updated")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.update_process.return_value = {"id": "proc1", "name": "proc_updated"}
            update_process(option_list)
            mock_client_instance.update_process.assert_called_once()
            mock_stdout.assert_called_once_with("Updated process detail: \n{'id': 'proc1', 'name': 'proc_updated'}")

    def test_get_process_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("process_id", ["--process-id"], "", True), "proc1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.get_process.return_value = {"id": "proc1"}
            get_process(option_list)
            mock_client_instance.get_process.assert_called_once_with(
                process_id="proc1", process_name=None, revision="0", version=0, allow_drafts=True
            )
            mock_stdout.assert_called_once_with("Process detail: \n{'id': 'proc1'}")

    def test_list_processes_success(self):
        option_list = [(Option("project_id", ["--project-id"], "", True), "proj123")]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.list_processes.return_value = [{"id": "proc1"}]
            list_processes(option_list)
            mock_client_instance.list_processes.assert_called_once_with(
                id=None, name=None, status=None, start="0", count="100", allow_draft=True
            )
            mock_stdout.assert_called_once_with("Process list: \n[{'id': 'proc1'}]")

    def test_list_processes_instances_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("process_id", ["--process-id"], "", True), "proc1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.list_process_instances.return_value = [{"id": "inst1"}]
            list_processes_instances(option_list)
            mock_client_instance.list_process_instances.assert_called_once_with(
                process_id="proc1", is_active=True, start="0", count="10"
            )
            mock_stdout.assert_called_once_with("Process instances list: \n[{'id': 'inst1'}]")

    def test_delete_process_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("process_id", ["--process-id"], "", True), "proc1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.delete_process.return_value = {"status": "deleted"}
            delete_process(option_list)
            mock_client_instance.delete_process.assert_called_once_with(
                process_id="proc1", process_name=None
            )
            mock_stdout.assert_called_once_with("Delete process result: \n{'status': 'deleted'}")

    def test_publish_process_revision_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("process_id", ["--process-id"], "", True), "proc1"),
            (Option("revision", ["--revision"], "", True), "1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.publish_process_revision.return_value = {"status": "published"}
            publish_process_revision(option_list)
            mock_client_instance.publish_process_revision.assert_called_once_with(
                process_id="proc1", process_name=None, revision="1"
            )
            mock_stdout.assert_called_once_with("Published process revision detail: \n{'status': 'published'}")

    # Task-related tests
    def test_create_task_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("name", ["--name"], "", True), "task1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.create_task.return_value = {"id": "task1"}
            create_task(option_list)
            mock_client_instance.create_task.assert_called_once()
            mock_stdout.assert_called_once_with("Created task detail: \n{'id': 'task1'}")

    def test_get_task_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("task_id", ["--task-id"], "", True), "task1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.get_task.return_value = {"id": "task1"}
            get_task(option_list)
            mock_client_instance.get_task.assert_called_once_with(
                task_id="task1", task_name=None
            )
            mock_stdout.assert_called_once_with("Task detail: \n{'id': 'task1'}")

    def test_list_tasks_success(self):
        option_list = [(Option("project_id", ["--project-id"], "", True), "proj123")]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.list_tasks.return_value = [{"id": "task1"}]
            list_tasks(option_list)
            mock_client_instance.list_tasks.assert_called_once_with(
                id=None, start="0", count="100", allow_drafts=True
            )
            mock_stdout.assert_called_once_with("Task list: \n[{'id': 'task1'}]")

    def test_update_task_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("task_id", ["--task-id"], "", True), "task1"),
            (Option("name", ["--name"], "", True), "task_updated")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.update_task.return_value = {"id": "task1", "name": "task_updated"}
            update_task(option_list)
            mock_client_instance.update_task.assert_called_once()
            mock_stdout.assert_called_once_with("Updated task detail: \n{'id': 'task1', 'name': 'task_updated'}")

    def test_delete_task_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("task_id", ["--task-id"], "", True), "task1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.delete_task.return_value = {"status": "deleted"}
            delete_task(option_list)
            mock_client_instance.delete_task.assert_called_once_with(
                task_id="task1", task_name=None
            )
            mock_stdout.assert_called_once_with("Delete task result: \n{'status': 'deleted'}")

    def test_publish_task_revision_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("task_id", ["--task-id"], "", True), "task1"),
            (Option("revision", ["--revision"], "", True), "1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.publish_task_revision.return_value = {"status": "published"}
            publish_task_revision(option_list)
            mock_client_instance.publish_task_revision.assert_called_once_with(
                task_id="task1", task_name=None, revision="1"
            )
            mock_stdout.assert_called_once_with("Published task revision detail: \n{'status': 'published'}")

    def test_start_instance_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("process_name", ["--process-name"], "", True), "proc1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.start_instance.return_value = {"id": "inst1"}
            start_instance(option_list)
            mock_client_instance.start_instance.assert_called_once_with(
                process_name="proc1", subject=None, variables=None
            )
            mock_stdout.assert_called_once_with("Started instance detail: \n{'id': 'inst1'}")

    def test_abort_instance_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("instance_id", ["--instance-id"], "", True), "inst1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.abort_instance.return_value = {"status": "aborted"}
            abort_instance(option_list)
            mock_client_instance.abort_instance.assert_called_once_with(
                instance_id="inst1"
            )
            mock_stdout.assert_called_once_with("Abort instance result: \n{'status': 'aborted'}")

    def test_get_instance_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("instance_id", ["--instance-id"], "", True), "inst1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
             patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.get_instance.return_value = {"id": "inst1"}
            get_instance(option_list)
            mock_client_instance.get_instance.assert_called_once_with(
                instance_id="inst1"
            )
            mock_stdout.assert_called_once_with("Instance detail: \n{'id': 'inst1'}")

    def test_get_thread_information_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("thread_id", ["--thread-id"], "", True), "thread1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
                patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.get_thread_information.return_value = {"id": "thread1"}
            get_thread_information(option_list)
            mock_client_instance.get_thread_information.assert_called_once_with(
                thread_id="thread1"
            )
            mock_stdout.assert_called_once_with("Thread information: \n{'id': 'thread1'}")

    def test_send_user_signal_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("instance_id", ["--instance-id"], "", True), "inst1"),
            (Option("signal_name", ["--signal-name"], "", True), "signal1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
                patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.send_user_signal.return_value = {"status": "sent"}
            send_user_signal(option_list)
            mock_client_instance.send_user_signal.assert_called_once_with(
                instance_id="inst1", signal_name="signal1"
            )
            mock_stdout.assert_called_once_with("Send user signal result: \n{'status': 'sent'}")

    def test_create_kb_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("name", ["--name"], "", True), "kb1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
                patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.create_kb.return_value = {"id": "kb1"}
            create_kb(option_list)
            mock_client_instance.create_kb.assert_called_once_with(
                name="kb1", artifacts=None, metadata=None
            )
            mock_stdout.assert_called_once_with("Created knowledge base detail: \n{'id': 'kb1'}")

    def test_get_kb_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("kb_id", ["--kb-id"], "", True), "kb1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
                patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.get_kb.return_value = {"id": "kb1"}
            get_kb(option_list)
            mock_client_instance.get_kb.assert_called_once_with(
                kb_id="kb1", kb_name=None
            )
            mock_stdout.assert_called_once_with("Knowledge base detail: \n{'id': 'kb1'}")

    def test_list_kbs_success(self):
        option_list = [(Option("project_id", ["--project-id"], "", True), "proj123")]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
                patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.list_kbs.return_value = [{"id": "kb1"}]
            list_kbs(option_list)
            mock_client_instance.list_kbs.assert_called_once_with(
                name=None, start="0", count="100"
            )
            mock_stdout.assert_called_once_with("Knowledge base list: \n[{'id': 'kb1'}]")

    def test_delete_kb_success(self):
        option_list = [
            (Option("project_id", ["--project-id"], "", True), "proj123"),
            (Option("kb_id", ["--kb-id"], "", True), "kb1")
        ]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
                patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.delete_kb.return_value = {"status": "deleted"}
            delete_kb(option_list)
            mock_client_instance.delete_kb.assert_called_once_with(
                kb_id="kb1", kb_name=None
            )
            mock_stdout.assert_called_once_with("Delete knowledge base result: \n{'status': 'deleted'}")

    def test_list_jobs_success(self):
        option_list = [(Option("project_id", ["--project-id"], "", True), "proj123")]
        with patch('pygeai.cli.commands.lab.ai_lab.AgenticProcessClient') as mock_client, \
                patch('pygeai.cli.commands.lab.ai_lab.Console.write_stdout') as mock_stdout:
            mock_client_instance = MagicMock()
            mock_client.return_value = mock_client_instance
            mock_client_instance.list_jobs.return_value = [{"id": "job1"}]
            list_jobs(option_list)
            mock_client_instance.list_jobs.assert_called_once_with(
                start="0", count="100", topic=None, token=None, name=None
            )
            mock_stdout.assert_called_once_with("Job list: \n[{'id': 'job1'}]")



from unittest import TestCase
from pygeai.cli.commands.lab.common import (
    get_agent_data_prompt_inputs,
    get_agent_data_prompt_outputs,
    get_agent_data_prompt_examples,
    get_tool_parameters
)
from pygeai.core.common.exceptions import WrongArgumentError


class TestLabCommon(TestCase):
    """
    python -m unittest pygeai.tests.cli.commands.lab.test_common.TestLabCommon
    """

    def test_get_agent_data_prompt_inputs_valid(self):
        input_list = ["input1", "input2"]
        result = get_agent_data_prompt_inputs(input_list)
        self.assertEqual(result, input_list)

    def test_get_agent_data_prompt_inputs_invalid_non_string(self):
        input_list = ["input1", 123]  # Non-string element
        with self.assertRaises(WrongArgumentError) as context:
            get_agent_data_prompt_inputs(input_list)
        self.assertIn("Inputs must be a list of strings", str(context.exception))

    def test_get_agent_data_prompt_inputs_empty(self):
        input_list = []
        result = get_agent_data_prompt_inputs(input_list)
        self.assertEqual(result, [])

    def test_get_agent_data_prompt_outputs_valid(self):
        output_list = [
            {"key": "output1", "description": "Description for output1"},
            {"key": "output2", "description": "Description for output2"}
        ]
        result = get_agent_data_prompt_outputs(output_list)
        self.assertEqual(result, output_list)

    def test_get_agent_data_prompt_outputs_invalid_missing_key(self):
        output_list = [
            {"description": "Description without key"}  # Missing 'key'
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_agent_data_prompt_outputs(output_list)
        self.assertIn("Each output must be in JSON format", str(context.exception))

    def test_get_agent_data_prompt_outputs_invalid_type(self):
        output_list = [
            "not a dict"  # Not a dictionary
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_agent_data_prompt_outputs(output_list)
        self.assertIn("Each output must be in JSON format", str(context.exception))

    def test_get_agent_data_prompt_outputs_empty(self):
        output_list = []
        result = get_agent_data_prompt_outputs(output_list)
        self.assertEqual(result, [])

    def test_get_agent_data_prompt_examples_valid(self):
        example_list = [
            {"inputData": "Input example 1", "output": '{"result": "Output 1"}'},
            {"inputData": "Input example 2", "output": '{"result": "Output 2"}'}
        ]
        result = get_agent_data_prompt_examples(example_list)
        self.assertEqual(result, example_list)

    def test_get_agent_data_prompt_examples_invalid_missing_field(self):
        example_list = [
            {"inputData": "Input without output"}  # Missing 'output'
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_agent_data_prompt_examples(example_list)
        self.assertIn("Each example must be in JSON format", str(context.exception))

    def test_get_agent_data_prompt_examples_invalid_type(self):
        example_list = [
            "not a dict"  # Not a dictionary
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_agent_data_prompt_examples(example_list)
        self.assertIn("Each example must be in JSON format", str(context.exception))

    def test_get_agent_data_prompt_examples_empty(self):
        example_list = []
        result = get_agent_data_prompt_examples(example_list)
        self.assertEqual(result, [])

    def test_get_tool_parameters_valid_regular(self):
        param_list = [
            {"key": "param1", "dataType": "String", "description": "Param 1 description", "isRequired": True}
        ]
        result = get_tool_parameters(param_list)
        self.assertEqual(result, param_list)

    def test_get_tool_parameters_valid_config(self):
        param_list = [
            {
                "key": "config1",
                "dataType": "String",
                "description": "Config 1 description",
                "isRequired": True,
                "type": "config",
                "fromSecret": False,
                "value": "config_value"
            }
        ]
        result = get_tool_parameters(param_list)
        self.assertEqual(result, param_list)

    def test_get_tool_parameters_invalid_missing_field(self):
        param_list = [
            {"key": "param1", "dataType": "String", "isRequired": True}  # Missing 'description'
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_tool_parameters(param_list)
        self.assertIn("Each parameter must contain 'key', 'dataType', 'description', and 'isRequired'", str(context.exception))

    def test_get_tool_parameters_invalid_key_type(self):
        param_list = [
            {"key": 123, "dataType": "String", "description": "Param description", "isRequired": True}  # 'key' not string
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_tool_parameters(param_list)
        self.assertIn("Parameter 'key' must be a string", str(context.exception))

    def test_get_tool_parameters_invalid_data_type(self):
        param_list = [
            {"key": "param1", "dataType": 123, "description": "Param description", "isRequired": True}  # 'dataType' not string
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_tool_parameters(param_list)
        self.assertIn("Parameter 'dataType' must be a string", str(context.exception))

    def test_get_tool_parameters_invalid_description_type(self):
        param_list = [
            {"key": "param1", "dataType": "String", "description": 123, "isRequired": True}  # 'description' not string
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_tool_parameters(param_list)
        self.assertIn("Parameter 'description' must be a string", str(context.exception))

    def test_get_tool_parameters_invalid_is_required_type(self):
        param_list = [
            {"key": "param1", "dataType": "String", "description": "Param description", "isRequired": "True"}  # 'isRequired' not bool
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_tool_parameters(param_list)
        self.assertIn("Parameter 'isRequired' must be a boolean", str(context.exception))

    def test_get_tool_parameters_invalid_type_value(self):
        param_list = [
            {
                "key": "param1",
                "dataType": "String",
                "description": "Param description",
                "isRequired": True,
                "type": "invalid"  # 'type' not 'config'
            }
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_tool_parameters(param_list)
        self.assertIn("Parameter 'type' must be 'config' if present", str(context.exception))

    def test_get_tool_parameters_invalid_from_secret_type(self):
        param_list = [
            {
                "key": "param1",
                "dataType": "String",
                "description": "Param description",
                "isRequired": True,
                "type": "config",
                "fromSecret": "True"  # 'fromSecret' not bool
            }
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_tool_parameters(param_list)
        self.assertIn("Parameter 'fromSecret' must be a boolean if present", str(context.exception))

    def test_get_tool_parameters_invalid_value_type(self):
        param_list = [
            {
                "key": "param1",
                "dataType": "String",
                "description": "Param description",
                "isRequired": True,
                "type": "config",
                "value": 123  # 'value' not string
            }
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_tool_parameters(param_list)
        self.assertIn("Parameter 'value' must be a string if present", str(context.exception))

    def test_get_tool_parameters_invalid_format(self):
        param_list = [
            "not a dict"  # Not a dictionary
        ]
        with self.assertRaises(WrongArgumentError) as context:
            get_tool_parameters(param_list)
        self.assertIn("Each parameter must contain", str(context.exception))

    def test_get_tool_parameters_empty(self):
        param_list = []
        result = get_tool_parameters(param_list)
        self.assertEqual(result, [])


CREATE_PROCESS_V2 = "v2/processes"  # POST -> Create process
UPDATE_PROCESS_V2 = "v2/processes/{processId}"  # PUT -> Update process
UPSERT_PROCESS_V2 = "v2/processes/{processId}/upsert"  # PUT -> Update or insert process
GET_PROCESS_V2 = "v2/processes/{processId}"  # GET -> Get process
LIST_PROCESSES_V2 = "v2/processes"  # GET -> Get process list
LIST_PROCESS_INSTANCES_V2 = "v2/processes/{processId}/instances"  # GET -> Get process instances
DELETE_PROCESS_V2 = "v2/processes/{processId}"  # DELETE -> Get process
PUBLISH_PROCESS_REVISION_V2 = "v2/processes/{processId}/publish-revision"  # POST -> Publish process revision
CREATE_TASK_V2 = "v2/processes/tasks"  # POST -> Create task
UPDATE_TASK_V2 = "v2/processes/tasks/{taskId}"  # PUT -> Update task
UPSERT_TASK_V2 = "v2/processes/tasks/{taskId}/upsert"  # PUT -> Update or insert task
GET_TASK_V2 = "v2/processes/tasks/{taskId}"  # GET -> Get task
LIST_TASKS_V2 = "v2/processes/tasks"  # GET -> Get task
DELETE_TASK_V2 = "v2/processes/tasks/{taskId}"  # DELETE -> Get task
PUBLISH_TASK_REVISION_V2 = "v2/processes/tasks/{taskId}/publish-revision"  # POST -> Publish task revision
START_INSTANCE_V2 = "v2/processes/instances"  # POST -> Start process instance
ABORT_INSTANCE_V2 = "v2/processes/instances/{instanceId}/abort"  # POST -> Abort process instance
GET_INSTANCE_V2 = "v2/processes/instances/{instanceId}"  # GET -> Get process instance
GET_INSTANCE_HISTORY_V2 = "v2/processes/instances/{instanceId}/trace"  # GET -> Get process instance history
GET_THREAD_INFORMATION_V2 = "v2/threads/{threadId}"  # GET -> Get thread information
SEND_USER_SIGNAL_V2 = "v2/processes/instances/{instanceId}/signal"  # POST -> Send user signal
CREATE_ARTIFACT_TYPE_V1 = "v1/agent-studio/agentic-process-definition/artifact-types"  # POST -> Create artifact type
UPDATE_ARTIFACT_TYPE_V1 = "v1/agent-studio/agentic-process-definition/artifact-types/{idOrName}"  # PUT -> Update artifact type
GET_ARTIFACT_TYPE_V1 = "v1/agent-studio/agentic-process-definition/artifact-types/{idOrName}"  # GET -> Get artifact type
LIST_ARTIFACT_TYPES_V1 = "v1/agent-studio/agentic-process-definition/artifact-types"  # GET -> List artifact types
DELETE_ARTIFACT_TYPE_V1 = "v1/agent-studio/agentic-process-definition/artifact-types/{idOrName}"  # DELETE -> Delete artifact type
CREATE_KB_V1 = "v1/agent-studio/agentic-process-definition/kbs"  # POST-> Create Kb
GET_KB_V1 = "v1/agent-studio/agentic-process-definition/kbs/{kbId}"  # GET-> Get Kb
LIST_KBS_V1 = "v1/agent-studio/agentic-process-definition/kbs"  # GET-> Get Kb
DELETE_KB_V1 = "v1/agent-studio/agentic-process-definition/kbs/{kbId}"  # DELETE-> Delete Kb
LIST_JOBS_V1 = "v1/agent-studio/runtime/jobs"  # GET-> Get list of Jobs



import asyncio
import logging

from a2a.server.tasks.push_notification_config_store import (
    PushNotificationConfigStore,
)
from a2a.types import PushNotificationConfig


logger = logging.getLogger(__name__)


class InMemoryPushNotificationConfigStore(PushNotificationConfigStore):
    """In-memory implementation of PushNotificationConfigStore interface.

    Stores push notification configurations in memory
    """

    def __init__(self) -> None:
        """Initializes the InMemoryPushNotificationConfigStore."""
        self.lock = asyncio.Lock()
        self._push_notification_infos: dict[
            str, list[PushNotificationConfig]
        ] = {}

    async def set_info(
        self, task_id: str, notification_config: PushNotificationConfig
    ) -> None:
        """Sets or updates the push notification configuration for a task in memory."""
        async with self.lock:
            if task_id not in self._push_notification_infos:
                self._push_notification_infos[task_id] = []

            if notification_config.id is None:
                notification_config.id = task_id

            for config in self._push_notification_infos[task_id]:
                if config.id == notification_config.id:
                    self._push_notification_infos[task_id].remove(config)
                    break

            self._push_notification_infos[task_id].append(notification_config)

    async def get_info(self, task_id: str) -> list[PushNotificationConfig]:
        """Retrieves the push notification configuration for a task from memory."""
        async with self.lock:
            return self._push_notification_infos.get(task_id) or []

    async def delete_info(
        self, task_id: str, config_id: str | None = None
    ) -> None:
        """Deletes the push notification configuration for a task from memory."""
        async with self.lock:
            if config_id is None:
                config_id = task_id

            if task_id in self._push_notification_infos:
                configurations = self._push_notification_infos[task_id]
                if not configurations:
                    return

                for config in configurations:
                    if config.id == config_id:
                        configurations.remove(config)
                        break

                if len(configurations) == 0:
                    del self._push_notification_infos[task_id]



import unittest
from unittest.mock import patch, MagicMock
from pygeai.core.omni.managers import OmniParserManager
from pygeai.core.omni.models import OmniParserConfiguration
from pygeai.core.omni.responses import OmniParserResponse
from pygeai.core.common.exceptions import APIError


class TestOmniParserManager(unittest.TestCase):

    def setUp(self):
        self.manager = OmniParserManager()
        self.config = OmniParserConfiguration(
            file_path="test.pdf",
            start_page=1,
            end_page=5
        )

    @patch('pygeai.core.omni.managers.OmniParserClient')
    def test_process_file_success(self, mock_client_class):
        mock_client = MagicMock()
        mock_client_class.return_value = mock_client

        mock_response_data = {
            "status": "success",
            "parts": [{"text": "Sample", "type": "Text"}],
            "requestId": "123",
            "usage": {"total_cost": 0.01}
        }
        mock_client.process_file.return_value = mock_response_data

        manager = OmniParserManager()
        result = manager.process_file(self.config)

        self.assertIsInstance(result, OmniParserResponse)
        self.assertEqual(result.status, "success")

    @patch('pygeai.core.omni.managers.OmniParserClient')
    def test_process_file_with_error(self, mock_client_class):
        mock_client = MagicMock()
        mock_client_class.return_value = mock_client

        mock_response_data = {
            "error": {
                "id": 100,
                "code": "PROCESSING_ERROR",
                "message": "File processing failed"
            }
        }
        mock_client.process_file.return_value = mock_response_data

        manager = OmniParserManager()
        with self.assertRaises(APIError):
            manager.process_file(self.config)



import logging

from a2a.client.errors import (
    A2AClientInvalidArgsError,
    A2AClientInvalidStateError,
)
from a2a.server.events.event_queue import Event
from a2a.types import (
    Message,
    Task,
    TaskArtifactUpdateEvent,
    TaskState,
    TaskStatus,
    TaskStatusUpdateEvent,
)
from a2a.utils import append_artifact_to_task


logger = logging.getLogger(__name__)


class ClientTaskManager:
    """Helps manage a task's lifecycle during execution of a request.

    Responsible for retrieving, saving, and updating the `Task` object based on
    events received from the agent.
    """

    def __init__(
        self,
    ) -> None:
        """Initializes the `ClientTaskManager`."""
        self._current_task: Task | None = None
        self._task_id: str | None = None
        self._context_id: str | None = None

    def get_task(self) -> Task | None:
        """Retrieves the current task object, either from memory.

        If `task_id` is set, it returns `_current_task` otherwise None.

        Returns:
            The `Task` object if found, otherwise `None`.
        """
        if not self._task_id:
            logger.debug('task_id is not set, cannot get task.')
            return None

        return self._current_task

    def get_task_or_raise(self) -> Task:
        """Retrieves the current task object.

        Returns:
            The `Task` object.

        Raises:
            A2AClientInvalidStateError: If there is no current known Task.
        """
        if not (task := self.get_task()):
            # Note: The source of this error is either from bad client usage
            # or from the server sending invalid updates. It indicates that this
            # task manager has not consumed any information about a task, yet
            # the caller is attempting to retrieve the current state of the task
            # it expects to be present.
            raise A2AClientInvalidStateError('no current Task')
        return task

    async def save_task_event(
        self, event: Task | TaskStatusUpdateEvent | TaskArtifactUpdateEvent
    ) -> Task | None:
        """Processes a task-related event (Task, Status, Artifact) and saves the updated task state.

        Ensures task and context IDs match or are set from the event.

        Args:
            event: The task-related event (`Task`, `TaskStatusUpdateEvent`, or `TaskArtifactUpdateEvent`).

        Returns:
            The updated `Task` object after processing the event.

        Raises:
            ClientError: If the task ID in the event conflicts with the TaskManager's ID
                         when the TaskManager's ID is already set.
        """
        if isinstance(event, Task):
            if self._current_task:
                raise A2AClientInvalidArgsError(
                    'Task is already set, create new manager for new tasks.'
                )
            await self._save_task(event)
            return event
        task_id_from_event = (
            event.id if isinstance(event, Task) else event.task_id
        )
        if not self._task_id:
            self._task_id = task_id_from_event
        if not self._context_id:
            self._context_id = event.context_id

        logger.debug(
            'Processing save of task event of type %s for task_id: %s',
            type(event).__name__,
            task_id_from_event,
        )

        task = self._current_task
        if not task:
            task = Task(
                status=TaskStatus(state=TaskState.unknown),
                id=task_id_from_event,
                context_id=self._context_id if self._context_id else '',
            )
        if isinstance(event, TaskStatusUpdateEvent):
            logger.debug(
                'Updating task %s status to: %s',
                event.task_id,
                event.status.state,
            )
            if event.status.message:
                if not task.history:
                    task.history = [event.status.message]
                else:
                    task.history.append(event.status.message)
            if event.metadata:
                if not task.metadata:
                    task.metadata = {}
                task.metadata.update(event.metadata)
            task.status = event.status
        else:
            logger.debug('Appending artifact to task %s', task.id)
            append_artifact_to_task(task, event)
        self._current_task = task
        return task

    async def process(self, event: Event) -> Event:
        """Processes an event, updates the task state if applicable, stores it, and returns the event.

        If the event is task-related (`Task`, `TaskStatusUpdateEvent`, `TaskArtifactUpdateEvent`),
        the internal task state is updated and persisted.

        Args:
            event: The event object received from the agent.

        Returns:
            The same event object that was processed.
        """
        if isinstance(
            event, Task | TaskStatusUpdateEvent | TaskArtifactUpdateEvent
        ):
            await self.save_task_event(event)

        return event

    async def _save_task(self, task: Task) -> None:
        """Saves the given task to the `_current_task` and updated `_task_id` and `_context_id`.

        Args:
            task: The `Task` object to save.
        """
        logger.debug('Saving task with id: %s', task.id)
        self._current_task = task
        if not self._task_id:
            logger.info('New task created with id: %s', task.id)
            self._task_id = task.id
            self._context_id = task.context_id

    def update_with_message(self, message: Message, task: Task) -> Task:
        """Updates a task object adding a new message to its history.

        If the task has a message in its current status, that message is moved
        to the history first.

        Args:
            message: The new `Message` to add to the history.
            task: The `Task` object to update.

        Returns:
            The updated `Task` object (updated in-place).
        """
        if task.status.message:
            if task.history:
                task.history.append(task.status.message)
            else:
                task.history = [task.status.message]
            task.status.message = None
        if task.history:
            task.history.append(message)
        else:
            task.history = [message]
        self._current_task = task
        return task
